@@ -2,8 +2,6 @@
 // Copyright (C) 2016 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#include <QtCore/qsystemdetection.h>
-
 #if 0
 #pragma qt_class(QtCompilerDetection)
 #pragma qt_sync_skip_header_check
@@ -14,6 +12,8 @@
 #define QCOMPILERDETECTION_H
 
 #include <QtCore/qprocessordetection.h>
+#include <QtCore/qsystemdetection.h>
+#include <QtCore/qtconfiginclude.h>
 
 /*
    The compiler, must be one of: (Q_CC_x)
@@ -75,8 +75,9 @@
 #  define Q_UNREACHABLE_IMPL() __assume(0)
 #  define Q_DECL_EXPORT __declspec(dllexport)
 #  define Q_DECL_IMPORT __declspec(dllimport)
-#  define QT_MAKE_UNCHECKED_ARRAY_ITERATOR(x) stdext::make_unchecked_array_iterator(x) // Since _MSC_VER >= 1800
-#  define QT_MAKE_CHECKED_ARRAY_ITERATOR(x, N) stdext::make_checked_array_iterator(x, size_t(N)) // Since _MSC_VER >= 1500
+#  if _MSC_VER < 1938 // stdext is deprecated since VS 2022 17.8
+#    define QT_MAKE_CHECKED_ARRAY_ITERATOR(x, N) stdext::make_checked_array_iterator(x, size_t(N)) // Since _MSC_VER >= 1500
+#  endif
 #  define Q_COMPILER_COMPLAINS_ABOUT_RETURN_AFTER_UNREACHABLE
 
 #elif defined(__BORLANDC__) || defined(__TURBOC__)
@@ -849,6 +850,13 @@
 #    if _MSC_VER >= 1910
 #      define Q_COMPILER_CONSTEXPR
 #    endif
+// MSVC versions before 19.36 have a bug in C++20 comparison implementation.
+// This leads to ambiguities when resolving comparison operator overloads in
+// certain scenarios (the buggy MSVC versions were checked using our CI and
+// compiler explorer).
+#    if _MSC_VER < 1936
+#      define Q_COMPILER_LACKS_THREE_WAY_COMPARE_SYMMETRY
+#    endif
 #  endif /* __cplusplus */
 #endif // defined(Q_CC_MSVC) && !defined(Q_CC_CLANG)
 
@@ -1213,11 +1221,11 @@
 #endif
 #endif
 #ifndef Q_FALLTHROUGH
-#  if defined(Q_CC_GNU_ONLY) && Q_CC_GNU >= 700
+#  ifdef Q_CC_GNU
 #    define Q_FALLTHROUGH() __attribute__((fallthrough))
 #  else
 #    define Q_FALLTHROUGH() (void)0
-#endif
+#  endif
 #endif
 
 
@@ -1390,6 +1398,11 @@ QT_WARNING_DISABLE_MSVC(4530) /* C++ exception handler used, but unwind semantic
 #  endif
 #endif
 
+// libstdc++ shipped with gcc < 11 does not have a fix for defect LWG 3346
+#if __cplusplus >= 202002L && (!defined(_GLIBCXX_RELEASE) || _GLIBCXX_RELEASE >= 11)
+#  define QT_COMPILER_HAS_LWG3346
+#endif
+
 #if defined(__cplusplus) && __cplusplus >= 202002L // P0846 doesn't have a feature macro :/
 #  define QT_COMPILER_HAS_P0846
 #endif
