@@ -228,8 +228,8 @@ asm(
 // List of features present with -march=x86-64-v3 and not architecturally
 // implied by __AVX2__
 #    define ARCH_HASWELL_MACROS     \
-    (__AVX2__ && __BMI__ && __BMI2__ && __F16C__ && __FMA__ && __LZCNT__ && __POPCNT__)
-#    if ARCH_HASWELL_MACROS == 0
+    (__AVX2__ + __BMI__ + __BMI2__ + __F16C__ + __FMA__ + __LZCNT__ + __POPCNT__)
+#    if ARCH_HASWELL_MACROS != 7
 #      error "Please enable all x86-64-v3 extensions; you probably want to use -march=haswell or -march=x86-64-v3 instead of -mavx2"
 #    endif
 static_assert(ARCH_HASWELL_MACROS, "Undeclared identifiers indicate which features are missing.");
@@ -325,12 +325,19 @@ static const uint64_t qCompilerCpuFeatures = 0
 #if defined __ARM_NEON__
         | CpuFeatureNEON
 #endif
+#if !(defined(Q_OS_LINUX) && defined(Q_PROCESSOR_ARM_64))
+        // Yocto Project recipes enable Crypto extension for all ARMv8 configs,
+        // even for targets without the Crypto extension. That's wrong, but as
+        // the compiler never generates the code for them on their own, most
+        // code never notices the problem. But we would. By not setting the
+        // bits here, we force a runtime detection.
 #if defined __ARM_FEATURE_CRC32
         | CpuFeatureCRC32
 #endif
 #if defined __ARM_FEATURE_CRYPTO
         | CpuFeatureAES
 #endif
+#endif // Q_OS_LINUX && Q_PROCESSOR_ARM64
 #if defined __mips_dsp
         | CpuFeatureDSP
 #endif
