@@ -16,51 +16,50 @@ class QJniObjectPrivate;
 
 class Q_CORE_EXPORT QJniObject
 {
+    friend class QJniArrayBase;
+
     template <typename ...Args>
     struct LocalFrame {
-        QJniEnvironment env;
+        mutable JNIEnv *env;
         bool hasFrame = false;
-        ~LocalFrame() {
+        explicit LocalFrame(JNIEnv *env = nullptr) noexcept
+            : env(env)
+        {
+        }
+        ~LocalFrame()
+        {
             if (hasFrame)
                 env->PopLocalFrame(nullptr);
         }
         template <typename T>
-        auto newLocalRef(QJniObject &&object) {
+        auto newLocalRef(jobject object)
+        {
             if (!hasFrame) {
-                if (env->PushLocalFrame(sizeof...(Args)) < 0)
+                if (jniEnv()->PushLocalFrame(sizeof...(Args)) < 0)
                     return T{}; // JVM is out of memory, avoid making matters worse
                 hasFrame = true;
             }
-            return static_cast<T>(env->NewLocalRef(object.template object<T>()));
-        }
-        JNIEnv *jniEnv() const { return env.jniEnv(); }
-        bool checkAndClearExceptions() { return env.checkAndClearExceptions(); }
-
-        template <typename T>
-        auto convertToJni(T &&value)
-        {
-            using Type = q20::remove_cvref_t<T>;
-            if constexpr (std::is_same_v<Type, QString>) {
-                return newLocalRef<jstring>(QJniObject::fromString(value));
-            } else if constexpr (std::is_base_of_v<QJniObject, Type>) {
-                return value.object();
-            } else {
-                return static_cast<T &&>(value);
-            }
+            return static_cast<T>(jniEnv()->NewLocalRef(object));
         }
         template <typename T>
-        auto convertFromJni(QJniObject &&object)
+        auto newLocalRef(const QJniObject &object)
         {
-            using Type = q20::remove_cvref_t<T>;
-            if constexpr (std::is_same_v<Type, QString>) {
-                return object.toString();
-            } else if constexpr (std::is_base_of_v<QJniObject, Type>
-                              && !std::is_same_v<QJniObject, Type>) {
-                return T{std::move(object)};
-            } else {
-                return std::move(object);
-            }
+            return newLocalRef<T>(object.template object<T>());
         }
+        JNIEnv *jniEnv() const
+        {
+            if (!env)
+                env = QJniEnvironment::getJniEnv();
+            return env;
+        }
+        bool checkAndClearExceptions()
+        {
+            return env ? QJniEnvironment::checkAndClearExceptions(env) : false;
+        }
+        template <typename T>
+        auto convertToJni(T &&value);
+        template <typename T>
+        auto convertFromJni(QJniObject &&object);
     };
 public:
     QJniObject();
@@ -72,12 +71,17 @@ public:
 #endif
         >
     explicit QJniObject(const char *className, Args &&...args)
-        : QJniObject(Qt::Uninitialized)
+        : QJniObject(LocalFrame<Args...>{}, className, std::forward<Args>(args)...)
     {
-        LocalFrame<Args...> localFrame;
-        *this = QJniObject(className, QtJniTypes::constructorSignature<Args...>().data(),
-                           localFrame.convertToJni(std::forward<Args>(args))...);
     }
+private:
+    template<typename ...Args>
+    explicit QJniObject(LocalFrame<Args...> localFrame, const char *className, Args &&...args)
+        : QJniObject(className, QtJniTypes::constructorSignature<Args...>().data(),
+                     localFrame.convertToJni(std::forward<Args>(args))...)
+    {
+    }
+public:
     explicit QJniObject(jclass clazz);
     explicit QJniObject(jclass clazz, const char *signature, ...);
     template<typename ...Args
@@ -119,12 +123,12 @@ public:
 
     template <typename Ret, typename ...Args
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<Ret> = true
+        , QtJniTypes::IfValidFieldType<Ret> = true
 #endif
     >
     auto callMethod(const char *methodName, const char *signature, Args &&...args) const
     {
-        LocalFrame<Args...> frame;
+        LocalFrame<Args...> frame(jniEnv());
         if constexpr (QtJniTypes::isObjectType<Ret>()) {
             return frame.template convertFromJni<Ret>(callObjectMethod(methodName, signature,
                                                 frame.convertToJni(std::forward<Args>(args))...));
@@ -151,7 +155,7 @@ public:
 
     template <typename Ret, typename ...Args
 #ifndef Q_QDOC
-        , QtJniTypes::ValidSignatureTypes<Ret, Args...> = true
+        , QtJniTypes::IfValidSignatureTypes<Ret, Args...> = true
 #endif
     >
     auto callMethod(const char *methodName, Args &&...args) const
@@ -162,14 +166,16 @@ public:
 
     template <typename Ret, typename ...Args
 #ifndef Q_QDOC
-        , QtJniTypes::ValidSignatureTypes<Ret, Args...> = true
+        , QtJniTypes::IfValidSignatureTypes<Ret, Args...> = true
 #endif
     >
     QJniObject callObjectMethod(const char *methodName, Args &&...args) const
     {
         QtJniTypes::assertObjectType<Ret>();
         constexpr auto signature = QtJniTypes::methodSignature<Ret, Args...>();
-        return callObjectMethod(methodName, signature.data(), std::forward<Args>(args)...);
+        LocalFrame<Args...> frame(jniEnv());
+        return frame.template convertFromJni<Ret>(callObjectMethod(methodName, signature,
+                                            frame.convertToJni(std::forward<Args>(args))...));
     }
 
     QJniObject callObjectMethod(const char *methodName, const char *signature, ...) const;
@@ -177,23 +183,23 @@ public:
     template <typename Ret, typename ...Args>
     static auto callStaticMethod(const char *className, const char *methodName, const char *signature, Args &&...args)
     {
-        QJniEnvironment env;
-        jclass clazz = QJniObject::loadClass(className, env.jniEnv());
+        JNIEnv *env = QJniEnvironment::getJniEnv();
+        jclass clazz = QJniObject::loadClass(className, env);
         return callStaticMethod<Ret>(clazz, methodName, signature, std::forward<Args>(args)...);
     }
 
     template <typename Ret, typename ...Args>
     static auto callStaticMethod(jclass clazz, const char *methodName, const char *signature, Args &&...args)
     {
-        QJniEnvironment env;
-        jmethodID id = clazz ? getMethodID(env.jniEnv(), clazz, methodName, signature, true)
+        JNIEnv *env = QJniEnvironment::getJniEnv();
+        jmethodID id = clazz ? getMethodID(env, clazz, methodName, signature, true)
                              : 0;
         return callStaticMethod<Ret>(clazz, id, std::forward<Args>(args)...);
     }
 
     template <typename Ret, typename ...Args
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<Ret> = true
+        , QtJniTypes::IfValidFieldType<Ret> = true
 #endif
     >
     static auto callStaticMethod(jclass clazz, jmethodID methodId, Args &&...args)
@@ -224,19 +230,22 @@ public:
 
     template <typename Ret, typename ...Args
 #ifndef Q_QDOC
-        , QtJniTypes::ValidSignatureTypes<Ret, Args...> = true
+        , QtJniTypes::IfValidSignatureTypes<Ret, Args...> = true
 #endif
     >
     static auto callStaticMethod(const char *className, const char *methodName, Args &&...args)
     {
-        QJniEnvironment env;
-        jclass clazz = QJniObject::loadClass(className, env.jniEnv());
-        return callStaticMethod<Ret>(clazz, methodName, std::forward<Args>(args)...);
+        JNIEnv *env = QJniEnvironment::getJniEnv();
+        jclass clazz = QJniObject::loadClass(className, env);
+        const jmethodID id = clazz ? getMethodID(env, clazz, methodName,
+                                         QtJniTypes::methodSignature<Ret, Args...>().data(), true)
+                                   : 0;
+        return callStaticMethod<Ret>(clazz, id, std::forward<Args>(args)...);
     }
 
     template <typename Ret, typename ...Args
 #ifndef Q_QDOC
-        , QtJniTypes::ValidSignatureTypes<Ret, Args...> = true
+        , QtJniTypes::IfValidSignatureTypes<Ret, Args...> = true
 #endif
     >
     static auto callStaticMethod(jclass clazz, const char *methodName, Args &&...args)
@@ -246,15 +255,15 @@ public:
     }
     template <typename Klass, typename Ret, typename ...Args
 #ifndef Q_QDOC
-        , QtJniTypes::ValidSignatureTypes<Ret, Args...> = true
+        , QtJniTypes::IfValidSignatureTypes<Ret, Args...> = true
 #endif
     >
     static auto callStaticMethod(const char *methodName, Args &&...args)
     {
-        QJniEnvironment env;
+        JNIEnv *env = QJniEnvironment::getJniEnv();
         const jclass clazz = QJniObject::loadClass(QtJniTypes::Traits<Klass>::className().data(),
-                                                   env.jniEnv());
-        const jmethodID id = clazz ? getMethodID(env.jniEnv(), clazz, methodName,
+                                                   env);
+        const jmethodID id = clazz ? getMethodID(env, clazz, methodName,
                                          QtJniTypes::methodSignature<Ret, Args...>().data(), true)
                                    : 0;
         return callStaticMethod<Ret>(clazz, id, std::forward<Args>(args)...);
@@ -271,7 +280,7 @@ public:
 
     template <typename Ret, typename ...Args
 #ifndef Q_QDOC
-        , QtJniTypes::ValidSignatureTypes<Ret, Args...> = true
+        , QtJniTypes::IfValidSignatureTypes<Ret, Args...> = true
 #endif
     >
     static QJniObject callStaticObjectMethod(const char *className, const char *methodName, Args &&...args)
@@ -285,7 +294,7 @@ public:
 
     template <typename Ret, typename ...Args
 #ifndef Q_QDOC
-        , QtJniTypes::ValidSignatureTypes<Ret, Args...> = true
+        , QtJniTypes::IfValidSignatureTypes<Ret, Args...> = true
 #endif
     >
     static QJniObject callStaticObjectMethod(jclass clazz, const char *methodName, Args &&...args)
@@ -299,12 +308,12 @@ public:
 
     template <typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     auto getField(const char *fieldName) const
     {
-        LocalFrame<T> frame;
+        LocalFrame<T> frame(jniEnv());
         if constexpr (QtJniTypes::isObjectType<T>()) {
             return frame.template convertFromJni<T>(getObjectField<T>(fieldName));
         } else {
@@ -322,7 +331,7 @@ public:
 
     template <typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     static auto getStaticField(const char *className, const char *fieldName)
@@ -340,7 +349,7 @@ public:
 
     template <typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     static auto getStaticField(jclass clazz, const char *fieldName)
@@ -363,7 +372,7 @@ public:
 
     template <typename Klass, typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     static auto getStaticField(const char *fieldName)
@@ -415,38 +424,36 @@ public:
 
     template <typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     void setField(const char *fieldName, T value)
     {
-        QJniEnvironment env;
         constexpr auto signature = QtJniTypes::fieldSignature<T>();
-        jfieldID id = getCachedFieldID(env.jniEnv(), fieldName, signature);
+        jfieldID id = getCachedFieldID(jniEnv(), fieldName, signature);
         if (id) {
-            setFieldForType<T>(env.jniEnv(), object(), id, value);
-            env.checkAndClearExceptions();
+            setFieldForType<T>(jniEnv(), object(), id, value);
+            QJniEnvironment::checkAndClearExceptions(jniEnv());
         }
     }
 
     template <typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     void setField(const char *fieldName, const char *signature, T value)
     {
-        QJniEnvironment env;
-        jfieldID id = getCachedFieldID(env.jniEnv(), fieldName, signature);
+        jfieldID id = getCachedFieldID(jniEnv(), fieldName, signature);
         if (id) {
-            setFieldForType<T>(env.jniEnv(), object(), id, value);
-            env.checkAndClearExceptions();
+            setFieldForType<T>(jniEnv(), object(), id, value);
+            QJniEnvironment::checkAndClearExceptions(jniEnv());
         }
     }
 
     template <typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     static void setStaticField(const char *className, const char *fieldName, T value)
@@ -468,46 +475,46 @@ public:
 
     template <typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     static void setStaticField(const char *className, const char *fieldName,
                                const char *signature, T value)
     {
-        QJniEnvironment env;
-        jclass clazz = QJniObject::loadClass(className, env.jniEnv());
+        JNIEnv *env = QJniEnvironment::getJniEnv();
+        jclass clazz = QJniObject::loadClass(className, env);
 
         if (!clazz)
             return;
 
-        jfieldID id = getCachedFieldID(env.jniEnv(), clazz, className, fieldName,
+        jfieldID id = getCachedFieldID(env, clazz, className, fieldName,
                                        signature, true);
         if (id) {
-            setStaticFieldForType<T>(env.jniEnv(), clazz, id, value);
-            env.checkAndClearExceptions();
+            setStaticFieldForType<T>(env, clazz, id, value);
+            QJniEnvironment::checkAndClearExceptions(env);
         }
     }
 
     template <typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     static void setStaticField(jclass clazz, const char *fieldName,
                                const char *signature, T value)
     {
-        QJniEnvironment env;
-        jfieldID id = getFieldID(env.jniEnv(), clazz, fieldName, signature, true);
+        JNIEnv *env = QJniEnvironment::getJniEnv();
+        jfieldID id = getFieldID(env, clazz, fieldName, signature, true);
 
         if (id) {
-            setStaticFieldForType<T>(env.jniEnv(), clazz, id, value);
-            env.checkAndClearExceptions();
+            setStaticFieldForType<T>(env, clazz, id, value);
+            QJniEnvironment::checkAndClearExceptions(env);
         }
     }
 
     template <typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     static void setStaticField(jclass clazz, const char *fieldName, T value)
@@ -517,7 +524,7 @@ public:
 
     template <typename Klass, typename T
 #ifndef Q_QDOC
-        , QtJniTypes::ValidFieldType<T> = true
+        , QtJniTypes::IfValidFieldType<T> = true
 #endif
     >
     static void setStaticField(const char *fieldName, T value)
@@ -534,7 +541,8 @@ public:
     // This function takes ownership of the jobject and releases the local ref. before returning.
     static QJniObject fromLocalRef(jobject lref);
 
-    template <typename T>
+    template <typename T,
+              std::enable_if_t<std::is_convertible_v<T, jobject>, bool> = true>
     QJniObject &operator=(T obj)
     {
         assign(static_cast<T>(obj));
@@ -543,6 +551,7 @@ public:
 
 protected:
     QJniObject(Qt::Initialization) {}
+    JNIEnv *jniEnv() const noexcept;
 
 private:
     static jclass loadClass(const QByteArray &className, JNIEnv *env);
@@ -580,46 +589,12 @@ private:
     friend bool operator==(const QJniObject &, const QJniObject &);
     friend bool operator!=(const QJniObject&, const QJniObject&);
 
-    template<typename T, typename = void>
-    struct IntegerTypeDetail
-    {
-        using type = T;
-    };
     template<typename T>
-    struct IntegerTypeDetail<T, typename std::enable_if_t<std::is_enum_v<T>>>
-    {
-        using type = std::underlying_type_t<T>;
-    };
-
-    template<typename Have, typename Want>
-    static constexpr bool likeIntegerType = std::is_same_v<Have, Want>
-                                         || std::is_same_v<typename IntegerTypeDetail<Have>::type, Want>;
-
-    template<typename T>
-    static constexpr void callMethodForType(JNIEnv *env, T &res, jobject obj,
-                                            jmethodID id, ...)
+    static constexpr void callMethodForType(JNIEnv *env, T &res, jobject obj, jmethodID id, ...)
     {
         va_list args = {};
         va_start(args, id);
-
-        if constexpr (std::is_same_v<T, jboolean>)
-            res = env->CallBooleanMethodV(obj, id, args);
-        else if constexpr (likeIntegerType<T, jbyte>)
-            res = T{env->CallByteMethodV(obj, id, args)};
-        else if constexpr (likeIntegerType<T, jchar>)
-            res = T{env->CallCharMethodV(obj, id, args)};
-        else if constexpr (likeIntegerType<T, jshort>)
-            res = T{env->CallShortMethodV(obj, id, args)};
-        else if constexpr (likeIntegerType<T, jint>)
-            res = T{env->CallIntMethodV(obj, id, args)};
-        else if constexpr (likeIntegerType<T, jlong>)
-            res = T{env->CallLongMethodV(obj, id, args)};
-        else if constexpr (std::is_same_v<T, jfloat>)
-            res = env->CallFloatMethodV(obj, id, args);
-        else if constexpr (std::is_same_v<T, jdouble>)
-            res = env->CallDoubleMethodV(obj, id, args);
-        else
-            QtJniTypes::staticAssertTypeMismatch();
+        QtJniTypes::Caller<T>::callMethodForType(env, res, obj, id, args);
         va_end(args);
     }
 
@@ -631,24 +606,7 @@ private:
             return;
         va_list args = {};
         va_start(args, id);
-        if constexpr (std::is_same_v<T, jboolean>)
-            res = env->CallStaticBooleanMethodV(clazz, id, args);
-        else if constexpr (likeIntegerType<T, jbyte>)
-            res = T{env->CallStaticByteMethodV(clazz, id, args)};
-        else if constexpr (likeIntegerType<T, jchar>)
-            res = T{env->CallStaticCharMethodV(clazz, id, args)};
-        else if constexpr (likeIntegerType<T, jshort>)
-            res = T{env->CallStaticShortMethodV(clazz, id, args)};
-        else if constexpr (likeIntegerType<T, jint>)
-            res = T{env->CallStaticIntMethodV(clazz, id, args)};
-        else if constexpr (likeIntegerType<T, jlong>)
-            res = T{env->CallStaticLongMethodV(clazz, id, args)};
-        else if constexpr (std::is_same_v<T, jfloat>)
-            res = env->CallStaticFloatMethodV(clazz, id, args);
-        else if constexpr (std::is_same_v<T, jdouble>)
-            res = env->CallStaticDoubleMethodV(clazz, id, args);
-        else
-            QtJniTypes::staticAssertTypeMismatch();
+        QtJniTypes::Caller<T>::callStaticMethodForType(env, res, clazz, id, args);
         va_end(args);
     }
 
@@ -664,105 +622,37 @@ private:
 
 
     template<typename T>
-    static constexpr void getFieldForType(JNIEnv *env, T &res, jobject obj,
-                                          jfieldID id)
+    static constexpr void getFieldForType(JNIEnv *env, T &res, jobject obj, jfieldID id)
     {
-        if constexpr (std::is_same_v<T, jboolean>)
-            res = env->GetBooleanField(obj, id);
-        else if constexpr (likeIntegerType<T, jbyte>)
-            res = T{env->GetByteField(obj, id)};
-        else if constexpr (likeIntegerType<T, jchar>)
-            res = T{env->GetCharField(obj, id)};
-        else if constexpr (likeIntegerType<T, jshort>)
-            res = T{env->GetShortField(obj, id)};
-        else if constexpr (likeIntegerType<T, jint>)
-            res = T{env->GetIntField(obj, id)};
-        else if constexpr (likeIntegerType<T, jlong>)
-            res = T{env->GetLongField(obj, id)};
-        else if constexpr (std::is_same_v<T, jfloat>)
-            res = env->GetFloatField(obj, id);
-        else if constexpr (std::is_same_v<T, jdouble>)
-            res = env->GetDoubleField(obj, id);
-        else
-            QtJniTypes::staticAssertTypeMismatch();
+        QtJniTypes::Caller<T>::getFieldForType(env, res, obj, id);
     }
 
     template<typename T>
-    static constexpr void getStaticFieldForType(JNIEnv *env, T &res, jclass clazz,
-                                                jfieldID id)
+    static constexpr void getStaticFieldForType(JNIEnv *env, T &res, jclass clazz, jfieldID id)
     {
-        if constexpr (std::is_same_v<T, jboolean>)
-            res = env->GetStaticBooleanField(clazz, id);
-        else if constexpr (likeIntegerType<T, jbyte>)
-            res = T{env->GetStaticByteField(clazz, id)};
-        else if constexpr (likeIntegerType<T, jchar>)
-            res = T{env->GetStaticCharField(clazz, id)};
-        else if constexpr (likeIntegerType<T, jshort>)
-            res = T{env->GetStaticShortField(clazz, id)};
-        else if constexpr (likeIntegerType<T, jint>)
-            res = T{env->GetStaticIntField(clazz, id)};
-        else if constexpr (likeIntegerType<T, jlong>)
-            res = T{env->GetStaticLongField(clazz, id)};
-        else if constexpr (std::is_same_v<T, jfloat>)
-            res = env->GetStaticFloatField(clazz, id);
-        else if constexpr (std::is_same_v<T, jdouble>)
-            res = env->GetStaticDoubleField(clazz, id);
-        else
-            QtJniTypes::staticAssertTypeMismatch();
+        QtJniTypes::Caller<T>::getStaticFieldForType(env, res, clazz, id);
     }
 
     template<typename T>
-    static constexpr void setFieldForType(JNIEnv *env, jobject obj,
-                                          jfieldID id, T value)
+    static constexpr void setFieldForType(JNIEnv *env, jobject obj, jfieldID id, T value)
     {
-        LocalFrame<T> frame;
-        if constexpr (std::is_same_v<T, jboolean>)
-            env->SetBooleanField(obj, id, value);
-        else if constexpr (likeIntegerType<T, jbyte>)
-            env->SetByteField(obj, id, static_cast<jbyte>(value));
-        else if constexpr (likeIntegerType<T, jchar>)
-            env->SetCharField(obj, id, static_cast<jchar>(value));
-        else if constexpr (likeIntegerType<T, jshort>)
-            env->SetShortField(obj, id, static_cast<jshort>(value));
-        else if constexpr (likeIntegerType<T, jint>)
-            env->SetIntField(obj, id, static_cast<jint>(value));
-        else if constexpr (likeIntegerType<T, jlong>)
-            env->SetLongField(obj, id, static_cast<jlong>(value));
-        else if constexpr (std::is_same_v<T, jfloat>)
-            env->SetFloatField(obj, id, value);
-        else if constexpr (std::is_same_v<T, jdouble>)
-            env->SetDoubleField(obj, id, value);
-        else if constexpr (QtJniTypes::isObjectType<T>())
+        if constexpr (QtJniTypes::isObjectType<T>()) {
+            LocalFrame<T> frame(env);
             env->SetObjectField(obj, id, static_cast<jobject>(frame.convertToJni(value)));
-        else
-            QtJniTypes::staticAssertTypeMismatch();
+        } else {
+            QtJniTypes::Caller<T>::setFieldForType(env, obj, id, value);
+        }
     }
 
     template<typename T>
-    static constexpr void setStaticFieldForType(JNIEnv *env, jclass clazz,
-                                          jfieldID id, T value)
+    static constexpr void setStaticFieldForType(JNIEnv *env, jclass clazz, jfieldID id, T value)
     {
-        LocalFrame<T> frame;
-        if constexpr (std::is_same_v<T, jboolean>)
-            env->SetStaticBooleanField(clazz, id, value);
-        else if constexpr (likeIntegerType<T, jbyte>)
-            env->SetStaticByteField(clazz, id, static_cast<jbyte>(value));
-        else if constexpr (likeIntegerType<T, jchar>)
-            env->SetStaticCharField(clazz, id, static_cast<jchar>(value));
-        else if constexpr (likeIntegerType<T, jshort>)
-            env->SetStaticShortField(clazz, id, static_cast<jshort>(value));
-        else if constexpr (likeIntegerType<T, jint>)
-            env->SetStaticIntField(clazz, id, static_cast<jint>(value));
-        else if constexpr (likeIntegerType<T, jlong>)
-            env->SetStaticLongField(clazz, id, static_cast<jlong>(value));
-        else if constexpr (std::is_same_v<T, jfloat>)
-            env->SetStaticFloatField(clazz, id, value);
-        else if constexpr (std::is_same_v<T, jdouble>)
-            env->SetStaticDoubleField(clazz, id, value);
-        else if constexpr (QtJniTypes::isObjectType<T>())
+        if constexpr (QtJniTypes::isObjectType<T>()) {
+            LocalFrame<T> frame(env);
             env->SetStaticObjectField(clazz, id, static_cast<jobject>(frame.convertToJni(value)));
-        else
-            QtJniTypes::staticAssertTypeMismatch();
+        } else {
+            QtJniTypes::Caller<T>::setStaticFieldForType(env, clazz, id, value);
+        }
     }
 
     friend QJniObjectPrivate;
@@ -779,6 +669,201 @@ inline bool operator!=(const QJniObject &obj1, const QJniObject &obj2)
     return !obj1.isSameObject(obj2);
 }
 
+namespace QtJniTypes {
+struct JObjectBase
+{
+    operator QJniObject() const { return m_object; }
+
+    bool isValid() const { return m_object.isValid(); }
+    jclass objectClass() const { return m_object.objectClass(); }
+    QString toString() const { return m_object.toString(); }
+
+    template <typename T = jobject>
+    T object() const {
+        return m_object.object<T>();
+    }
+
+protected:
+    JObjectBase() = default;
+    ~JObjectBase() = default;
+
+    Q_IMPLICIT JObjectBase(jobject object) : m_object(object) {}
+    Q_IMPLICIT JObjectBase(const QJniObject &object) : m_object(object) {}
+    Q_IMPLICIT JObjectBase(QJniObject &&object) noexcept : m_object(std::move(object)) {}
+
+    QJniObject m_object;
+};
+
+template<typename Type>
+class JObject : public JObjectBase
+{
+public:
+    using Class = Type;
+
+    JObject()
+        : JObjectBase{QJniObject(QtJniTypes::Traits<Class>::className())}
+    {}
+    Q_IMPLICIT JObject(jobject object) : JObjectBase(object) {}
+    Q_IMPLICIT JObject(const QJniObject &object) : JObjectBase(object) {}
+    Q_IMPLICIT JObject(QJniObject &&object) noexcept : JObjectBase(std::move(object)) {}
+
+    // base class destructor is protected, so need to provide all SMFs
+    JObject(const JObject &other) = default;
+    JObject(JObject &&other) noexcept = default;
+    JObject &operator=(const JObject &other) = default;
+    JObject &operator=(JObject &&other) noexcept = default;
+
+    ~JObject() = default;
+
+    template<typename Arg, typename ...Args
+            , std::enable_if_t<!std::is_same_v<Arg, JObject>, bool> = true
+            , IfValidSignatureTypes<Arg, Args...> = true
+    >
+    explicit JObject(Arg && arg, Args &&...args)
+        : JObjectBase{QJniObject(QtJniTypes::Traits<Class>::className(),
+                                 std::forward<Arg>(arg), std::forward<Args>(args)...)}
+    {}
+
+    // named constructors avoid ambiguities
+    static Type fromJObject(jobject object) { return Type(object); }
+    template <typename ...Args>
+    static Type construct(Args &&...args) { return Type(std::forward<Args>(args)...); }
+    static Type fromLocalRef(jobject lref) { return Type(QJniObject::fromLocalRef(lref)); }
+
+    static bool registerNativeMethods(std::initializer_list<JNINativeMethod> methods)
+    {
+        QJniEnvironment env;
+        return env.registerNativeMethods<Class>(methods);
+    }
+
+    // public API forwarding to QJniObject, with the implicit Class template parameter
+    template <typename Ret, typename ...Args
+#ifndef Q_QDOC
+        , QtJniTypes::IfValidSignatureTypes<Ret, Args...> = true
+#endif
+    >
+    static auto callStaticMethod(const char *name, Args &&...args)
+    {
+        return QJniObject::callStaticMethod<Class, Ret, Args...>(name,
+                                                                 std::forward<Args>(args)...);
+    }
+    template <typename T
+#ifndef Q_QDOC
+        , QtJniTypes::IfValidFieldType<T> = true
+#endif
+    >
+    static auto getStaticField(const char *field)
+    {
+        return QJniObject::getStaticField<Class, T>(field);
+    }
+    template <typename T
+#ifndef Q_QDOC
+        , QtJniTypes::IfValidFieldType<T> = true
+#endif
+    >
+    static void setStaticField(const char *field, T &&value)
+    {
+        QJniObject::setStaticField<Class, T>(field, std::forward<T>(value));
+    }
+
+    // keep only these overloads, the rest is made private
+    template <typename Ret, typename ...Args
+#ifndef Q_QDOC
+        , QtJniTypes::IfValidSignatureTypes<Ret, Args...> = true
+#endif
+    >
+    auto callMethod(const char *method, Args &&...args) const
+    {
+        return m_object.callMethod<Ret>(method, std::forward<Args>(args)...);
+    }
+    template <typename T
+#ifndef Q_QDOC
+        , QtJniTypes::IfValidFieldType<T> = true
+#endif
+    >
+    auto getField(const char *fieldName) const
+    {
+        return m_object.getField<T>(fieldName);
+    }
+
+    template <typename T
+#ifndef Q_QDOC
+        , QtJniTypes::IfValidFieldType<T> = true
+#endif
+    >
+    void setField(const char *fieldName, T &&value)
+    {
+        m_object.setField(fieldName, std::forward<T>(value));
+    }
+
+    QByteArray className() const {
+        return QtJniTypes::Traits<Class>::className().data();
+    }
+
+private:
+    friend bool comparesEqual(const JObject &lhs, const JObject &rhs) noexcept
+    { return lhs.m_object == rhs.m_object; }
+    Q_DECLARE_EQUALITY_COMPARABLE_LITERAL_TYPE(JObject);
+};
+}
+
+// This cannot be included earlier as QJniArray is a QJniObject subclass, but it
+// must be included so that we can implement QJniObject::LocalFrame conversion.
+QT_END_NAMESPACE
+#include <QtCore/qjniarray.h>
+QT_BEGIN_NAMESPACE
+
+template <typename ...Args>
+template <typename T>
+auto QJniObject::LocalFrame<Args...>::convertToJni(T &&value)
+{
+    using Type = q20::remove_cvref_t<T>;
+    if constexpr (std::is_same_v<Type, QString>) {
+        return newLocalRef<jstring>(QJniObject::fromString(value));
+    } else if constexpr (QtJniTypes::IsJniArray<Type>::value) {
+        return value.arrayObject();
+    } else if constexpr (QJniArrayBase::canConvert<T>) {
+        using QJniArrayType = decltype(QJniArrayBase::fromContainer(std::forward<T>(value)));
+        using ArrayType = decltype(std::declval<QJniArrayType>().arrayObject());
+        return newLocalRef<ArrayType>(QJniArrayBase::fromContainer(std::forward<T>(value)).template object<jobject>());
+    } else if constexpr (std::is_base_of_v<QJniObject, Type>
+                      || std::is_base_of_v<QtJniTypes::JObjectBase, Type>) {
+        return value.object();
+    } else {
+        return std::forward<T>(value);
+    }
+}
+
+template <typename ...Args>
+template <typename T>
+auto QJniObject::LocalFrame<Args...>::convertFromJni(QJniObject &&object)
+{
+    using Type = q20::remove_cvref_t<T>;
+    if constexpr (std::is_same_v<Type, QString>) {
+        return object.toString();
+    } else if constexpr (QtJniTypes::IsJniArray<Type>::value) {
+        return T(std::move(object));
+    } else if constexpr (QJniArrayBase::canConvert<Type>) {
+        // if we were to create a QJniArray from Type...
+        using QJniArrayType = decltype(QJniArrayBase::fromContainer(std::declval<Type>()));
+        // then that QJniArray would have elements of type
+        using ElementType = typename QJniArrayType::Type;
+        // construct a QJniArray from a jobject pointer of that type
+        return QJniArray<ElementType>(object.template object<jarray>()).toContainer();
+    } else if constexpr (std::is_array_v<Type>) {
+        using ElementType = std::remove_extent_t<Type>;
+        return QJniArray<ElementType>(std::move(object));
+    } else if constexpr (std::is_base_of_v<QJniObject, Type>
+                        && !std::is_same_v<QJniObject, Type>) {
+        return T{std::move(object)};
+    } else if constexpr (std::is_base_of_v<QtJniTypes::JObjectBase, Type>) {
+        return T{std::move(object)};
+    } else {
+        return std::move(object);
+    }
+}
+
+
 QT_END_NAMESPACE
 
 #endif
