@@ -11,7 +11,6 @@
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qjsondocument.h>
-#include <qmap.h>
 #include <qmutex.h>
 #include <qoperatingsystemversion.h>
 #include <qstringlist.h>
@@ -30,6 +29,8 @@
 
 #include <qtcore_tracepoints_p.h>
 
+#include <QtCore/q20map.h>
+
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -321,7 +322,7 @@ private:
     static inline QLibraryStore *instance();
 
     // all members and instance() are protected by qt_library_mutex
-    typedef QMap<QString, QLibraryPrivate *> LibraryMap;
+    typedef std::map<QString, QLibraryPrivate *> LibraryMap;
     LibraryMap libraryMap;
 };
 
@@ -341,9 +342,7 @@ inline void QLibraryStore::cleanup()
         return;
 
     // find any libraries that are still loaded but have a no one attached to them
-    LibraryMap::Iterator it = data->libraryMap.begin();
-    for (; it != data->libraryMap.end(); ++it) {
-        QLibraryPrivate *lib = it.value();
+    for (auto &[_, lib] : data->libraryMap) {
         if (lib->libraryRefCount.loadRelaxed() == 1) {
             if (lib->libraryUnloadCount.loadRelaxed() > 0) {
                 Q_ASSERT(lib->pHnd.loadRelaxed());
@@ -357,14 +356,13 @@ inline void QLibraryStore::cleanup()
                 lib->unload();
 #endif
             }
-            delete lib;
-            it.value() = nullptr;
+            delete std::exchange(lib, nullptr);
         }
     }
 
     // dump all objects that remain
     if (lcDebugLibrary().isDebugEnabled()) {
-        for (QLibraryPrivate *lib : std::as_const(data->libraryMap)) {
+        for (auto &[_, lib] : data->libraryMap) {
             if (lib)
                 qDebug(lcDebugLibrary)
                         << "On QtCore unload," << lib->fileName << "was leaked, with"
@@ -395,26 +393,34 @@ QLibraryStore *QLibraryStore::instance()
 inline QLibraryPrivate *QLibraryStore::findOrCreate(const QString &fileName, const QString &version,
                                                     QLibrary::LoadHints loadHints)
 {
+    auto lazyNewLib = [&] {
+        auto result = new QLibraryPrivate(fileName, version, loadHints);
+        result->libraryRefCount.ref();
+        return result;
+    };
+
+    if (fileName.isEmpty())   // request for empty d-pointer in QLibrary::setLoadHints();
+        return lazyNewLib();  // must return an independent (new) object
+
     QMutexLocker locker(&qt_library_mutex);
     QLibraryStore *data = instance();
 
+    if (Q_UNLIKELY(!data)) {
+        locker.unlock();
+        return lazyNewLib();
+    }
+
     QString mapName = version.isEmpty() ? fileName : fileName + u'\0' + version;
 
-    // check if this library is already loaded
-    QLibraryPrivate *lib = nullptr;
-    if (Q_LIKELY(data)) {
-        lib = data->libraryMap.value(mapName);
-        if (lib)
-            lib->mergeLoadHints(loadHints);
+    QLibraryPrivate *&lib = data->libraryMap[std::move(mapName)];
+    if (lib) {
+        // already loaded
+        lib->libraryRefCount.ref();
+        lib->mergeLoadHints(loadHints);
+    } else {
+        lib = lazyNewLib();
     }
-    if (!lib)
-        lib = new QLibraryPrivate(fileName, version, loadHints);
 
-    // track this library
-    if (Q_LIKELY(data) && !fileName.isEmpty())
-        data->libraryMap.insert(mapName, lib);
-
-    lib->libraryRefCount.ref();
     return lib;
 }
 
@@ -432,8 +438,9 @@ inline void QLibraryStore::releaseLibrary(QLibraryPrivate *lib)
     Q_ASSERT(lib->libraryUnloadCount.loadRelaxed() == 0);
 
     if (Q_LIKELY(data) && !lib->fileName.isEmpty()) {
-        qsizetype n = erase_if(data->libraryMap, [lib](LibraryMap::iterator it) {
-            return it.value() == lib;
+        using q20::erase_if;
+        const auto n = erase_if(data->libraryMap, [lib](const auto &e) {
+            return e.second == lib;
         });
         Q_ASSERT_X(n, "~QLibrary", "Did not find this library in the library map");
         Q_UNUSED(n);
