@@ -146,7 +146,24 @@ void QAdoptedThread::run()
     // this function should never be called
     qFatal("QAdoptedThread::run(): Internal error, this implementation should never be called.");
 }
+#endif
 
+QScopedScopeLevelCounter::QScopedScopeLevelCounter(QThreadData *threadData)
+    : threadData(threadData)
+{
+    ++threadData->scopeLevel;
+    qCDebug(lcDeleteLater) << "Increased" << threadData->thread
+                      << "scope level to" << threadData->scopeLevel;
+}
+
+QScopedScopeLevelCounter::~QScopedScopeLevelCounter()
+{
+    --threadData->scopeLevel;
+    qCDebug(lcDeleteLater) << "Decreased" << threadData->thread
+                      << "scope level to" << threadData->scopeLevel;
+}
+
+#if QT_CONFIG(thread)
 /*
   QThreadPrivate
 */
@@ -256,22 +273,21 @@ QThreadPrivate::~QThreadPrivate()
     documentation for terminate() and setTerminationEnabled() for
     detailed information.
 
-    From Qt 4.8 onwards, it is possible to deallocate objects that
-    live in a thread that has just ended, by connecting the
-    finished() signal to QObject::deleteLater().
+    You often want to deallocate objects that live in a thread when
+    a thread ends. To do this, connect the finished() signal to
+    QObject::deleteLater().
 
     Use wait() to block the calling thread, until the other thread
     has finished execution (or until a specified time has passed).
 
     QThread also provides static, platform independent sleep
     functions: sleep(), msleep(), and usleep() allow full second,
-    millisecond, and microsecond resolution respectively. These
-    functions were made public in Qt 5.0.
+    millisecond, and microsecond resolution respectively.
 
     \note wait() and the sleep() functions should be unnecessary in
     general, since Qt is an event-driven framework. Instead of
     wait(), consider listening for the finished() signal. Instead of
-    the sleep() functions, consider using QTimer.
+    the sleep() functions, consider using QChronoTimer.
 
     The static functions currentThreadId() and currentThread() return
     identifiers for the currently executing thread. The former
@@ -1000,6 +1016,10 @@ bool QThread::isInterruptionRequested() const
     return false;
 }
 
+void QThread::setTerminationEnabled(bool)
+{
+}
+
 // No threads: so we can just use static variables
 Q_CONSTINIT static QThreadData *data = nullptr;
 
