@@ -250,7 +250,15 @@ public:
         Half4,
         Half3,
         Half2,
-        Half
+        Half,
+        UShort4,
+        UShort3,
+        UShort2,
+        UShort,
+        SShort4,
+        SShort3,
+        SShort2,
+        SShort,
     };
 
     QRhiVertexInputAttribute() = default;
@@ -495,7 +503,7 @@ public:
             quint32 maybeSize;
             bool hasDynamicOffset;
         };
-        static const int MAX_TEX_SAMPLER_ARRAY_SIZE = 16;
+        static constexpr int MAX_TEX_SAMPLER_ARRAY_SIZE = 16;
         struct TextureAndOrSamplerData {
             int count;
             TextureAndSampler texSamplers[MAX_TEX_SAMPLER_ARRAY_SIZE];
@@ -535,7 +543,7 @@ public:
         }
     };
 
-    static const int LAYOUT_DESC_ENTRIES_PER_BINDING = 4;
+    static constexpr int LAYOUT_DESC_ENTRIES_PER_BINDING = 4;
 
     template<typename Output>
     static void serializeLayoutDescription(const QRhiShaderResourceBinding *first,
@@ -634,10 +642,14 @@ public:
     QRhiTexture *depthTexture() const { return m_depthTexture; }
     void setDepthTexture(QRhiTexture *texture) { m_depthTexture = texture; }
 
+    QRhiTexture *depthResolveTexture() const { return m_depthResolveTexture; }
+    void setDepthResolveTexture(QRhiTexture *tex) { m_depthResolveTexture = tex; }
+
 private:
     QVarLengthArray<QRhiColorAttachment, 8> m_colorAttachments;
     QRhiRenderBuffer *m_depthStencilBuffer = nullptr;
     QRhiTexture *m_depthTexture = nullptr;
+    QRhiTexture *m_depthResolveTexture = nullptr;
 };
 
 class Q_GUI_EXPORT QRhiTextureSubresourceUploadDescription
@@ -983,6 +995,15 @@ public:
     int sampleCount() const { return m_sampleCount; }
     void setSampleCount(int s) { m_sampleCount = s; }
 
+    struct ViewFormat {
+        QRhiTexture::Format format;
+        bool srgb;
+    };
+    ViewFormat readViewFormat() const { return m_readViewFormat; }
+    void setReadViewFormat(const ViewFormat &fmt) { m_readViewFormat = fmt; }
+    ViewFormat writeViewFormat() const { return m_writeViewFormat; }
+    void setWriteViewFormat(const ViewFormat &fmt) { m_writeViewFormat = fmt; }
+
     virtual bool create() = 0;
     virtual NativeTexture nativeTexture();
     virtual bool createFrom(NativeTexture src);
@@ -999,6 +1020,8 @@ protected:
     Flags m_flags;
     int m_arrayRangeStart = -1;
     int m_arrayRangeLength = -1;
+    ViewFormat m_readViewFormat = { UnknownFormat, false };
+    ViewFormat m_writeViewFormat = { UnknownFormat, false };
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QRhiTexture::Flags)
@@ -1162,7 +1185,8 @@ class Q_GUI_EXPORT QRhiTextureRenderTarget : public QRhiRenderTarget
 public:
     enum Flag {
         PreserveColorContents = 1 << 0,
-        PreserveDepthStencilContents = 1 << 1
+        PreserveDepthStencilContents = 1 << 1,
+        DoNotStoreDepthStencilContents = 1 << 2
     };
     Q_DECLARE_FLAGS(Flags, Flag)
 
@@ -1217,7 +1241,7 @@ public:
     virtual void updateResources(UpdateFlags flags = {}) = 0;
 
 protected:
-    static const int BINDING_PREALLOC = 12;
+    static constexpr int BINDING_PREALLOC = 12;
     QRhiShaderResourceBindings(QRhiImplementation *rhi);
     QVarLengthArray<QRhiShaderResourceBinding, BINDING_PREALLOC> m_bindings;
     size_t m_layoutDescHash = 0;
@@ -1237,6 +1261,11 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QRhiShaderResourceBindings::UpdateFlags)
 Q_GUI_EXPORT QDebug operator<<(QDebug, const QRhiShaderResourceBindings &);
 #endif
 
+// The proper name. Until it gets rolled out universally, have the better name
+// as a typedef. Eventually it should be reversed (the old name being a typedef
+// to the new one).
+using QRhiShaderResourceBindingSet = QRhiShaderResourceBindings;
+
 class Q_GUI_EXPORT QRhiGraphicsPipeline : public QRhiResource
 {
 public:
@@ -1787,7 +1816,8 @@ public:
         EnableDebugMarkers = 1 << 0,
         PreferSoftwareRenderer = 1 << 1,
         EnablePipelineCacheDataSave = 1 << 2,
-        EnableTimestamps = 1 << 3
+        EnableTimestamps = 1 << 3,
+        SuppressSmokeTestWarnings = 1 << 4
     };
     Q_DECLARE_FLAGS(Flags, Flag)
 
@@ -1839,7 +1869,9 @@ public:
         HalfAttributes,
         RenderToOneDimensionalTexture,
         ThreeDimensionalTextureMipmaps,
-        MultiView
+        MultiView,
+        TextureViewFormat,
+        ResolveDepthStencil
     };
 
     enum BeginFrameFlag {
@@ -1962,7 +1994,7 @@ public:
     const QRhiNativeHandles *nativeHandles();
     bool makeThreadLocalNativeContextCurrent();
 
-    static const int MAX_MIP_LEVELS = 16; // -> max width or height is 65536
+    static constexpr int MAX_MIP_LEVELS = 16; // -> max width or height is 65536
 
     void releaseCachedResources();
 
