@@ -39,11 +39,14 @@ static const QCssKnownValue properties[NumProperties - 1] = {
     { "-qt-background-role", QtBackgroundRole },
     { "-qt-block-indent", QtBlockIndent },
     { "-qt-fg-texture-cachekey", QtForegroundTextureCacheKey },
+    { "-qt-foreground", QtForeground },
     { "-qt-line-height-type", QtLineHeightType },
     { "-qt-list-indent", QtListIndent },
     { "-qt-list-number-prefix", QtListNumberPrefix },
     { "-qt-list-number-suffix", QtListNumberSuffix },
     { "-qt-paragraph-type", QtParagraphType },
+    { "-qt-stroke-color", QtStrokeColor },
+    { "-qt-stroke-width", QtStrokeWidth },
     { "-qt-style-features", QtStyleFeatures },
     { "-qt-table-type", QtTableType },
     { "-qt-user-state", QtUserState },
@@ -812,6 +815,10 @@ static BrushData parseBrushValue(const QCss::Value &v, const QPalette &pal)
     QStringList spreads;
     spreads << "pad"_L1 << "reflect"_L1 << "repeat"_L1;
 
+    int coordinateMode = -1;
+    QStringList coordinateModes;
+    coordinateModes << "logical"_L1 << "stretchtodevice"_L1 << "objectbounding"_L1 << "object"_L1;
+
     bool dependsOnThePalette = false;
     Parser parser(lst.at(1));
     while (parser.hasNext()) {
@@ -838,11 +845,12 @@ static BrushData parseBrushValue(const QCss::Value &v, const QPalette &pal)
             parser.next();
             QCss::Value value;
             (void)parser.parseTerm(&value);
-            if (attr.compare("spread"_L1, Qt::CaseInsensitive) == 0) {
+            if (attr.compare("spread"_L1, Qt::CaseInsensitive) == 0)
                 spread = spreads.indexOf(value.variant.toString());
-            } else {
+            else if (attr.compare("coordinatemode"_L1, Qt::CaseInsensitive) == 0)
+                coordinateMode = coordinateModes.indexOf(value.variant.toString());
+            else
                 vars[attr] = value.variant.toReal();
-            }
         }
         parser.skipSpace();
         (void)parser.test(COMMA);
@@ -851,7 +859,7 @@ static BrushData parseBrushValue(const QCss::Value &v, const QPalette &pal)
     if (gradType == 0) {
         QLinearGradient lg(vars.value("x1"_L1), vars.value("y1"_L1),
                            vars.value("x2"_L1), vars.value("y2"_L1));
-        lg.setCoordinateMode(QGradient::ObjectBoundingMode);
+        lg.setCoordinateMode(coordinateMode < 0 ? QGradient::ObjectBoundingMode : QGradient::CoordinateMode(coordinateMode));
         lg.setStops(stops);
         if (spread != -1)
             lg.setSpread(QGradient::Spread(spread));
@@ -865,7 +873,7 @@ static BrushData parseBrushValue(const QCss::Value &v, const QPalette &pal)
         QRadialGradient rg(vars.value("cx"_L1), vars.value("cy"_L1),
                            vars.value("radius"_L1), vars.value("fx"_L1),
                            vars.value("fy"_L1));
-        rg.setCoordinateMode(QGradient::ObjectBoundingMode);
+        rg.setCoordinateMode(coordinateMode < 0 ? QGradient::ObjectBoundingMode : QGradient::CoordinateMode(coordinateMode));
         rg.setStops(stops);
         if (spread != -1)
             rg.setSpread(QGradient::Spread(spread));
@@ -877,7 +885,7 @@ static BrushData parseBrushValue(const QCss::Value &v, const QPalette &pal)
 
     if (gradType == 2) {
         QConicalGradient cg(vars.value("cx"_L1), vars.value("cy"_L1), vars.value("angle"_L1));
-        cg.setCoordinateMode(QGradient::ObjectBoundingMode);
+        cg.setCoordinateMode(coordinateMode < 0 ? QGradient::ObjectBoundingMode : QGradient::CoordinateMode(coordinateMode));
         cg.setStops(stops);
         if (spread != -1)
             cg.setSpread(QGradient::Spread(spread));
@@ -979,9 +987,11 @@ void ValueExtractor::borderValue(const Declaration &decl, int *width, QCss::Bord
     }
 
      data.color = parseBrushValue(decl.d->values.at(i), pal);
-     *color = brushFromData(data.color, pal);
-     if (data.color.type != BrushData::DependsOnThePalette)
-         decl.d->parsed = QVariant::fromValue<BorderData>(data);
+    if (data.color.type != BrushData::Invalid) {
+        *color = brushFromData(data.color, pal);
+        if (data.color.type != BrushData::DependsOnThePalette)
+            decl.d->parsed = QVariant::fromValue<BorderData>(data);
+    }
 }
 
 static void parseShorthandBackgroundProperty(const QList<QCss::Value> &values, BrushData *brush, QString *image, Repeat *repeat, Qt::Alignment *alignment, const QPalette &pal)
