@@ -17,6 +17,8 @@
 #include <qbuffer.h>
 #include <qpair.h>
 #include <qdebug.h>
+#include <qspan.h>
+#include <qvarlengtharray.h>
 
 #ifndef QT_NO_SSL
 #    include <private/qsslsocket_p.h>
@@ -32,56 +34,35 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
-// Note: Only used from auto tests, normal usage is via QHttp1Configuration
-const int QHttpNetworkConnectionPrivate::defaultHttpChannelCount = 6;
-
 // The pipeline length. So there will be 4 requests in flight.
 const int QHttpNetworkConnectionPrivate::defaultPipelineLength = 3;
 // Only re-fill the pipeline if there's defaultRePipelineLength slots free in the pipeline.
 // This means that there are 2 requests in flight and 2 slots free that will be re-filled.
 const int QHttpNetworkConnectionPrivate::defaultRePipelineLength = 2;
 
-
-QHttpNetworkConnectionPrivate::QHttpNetworkConnectionPrivate(const QString &hostName,
-                                                             quint16 port, bool encrypt,
-                                                             QHttpNetworkConnection::ConnectionType type)
-: state(RunningState),
-  networkLayerState(Unknown),
-  hostName(hostName), port(port), encrypt(encrypt), delayIpv4(true)
-  , activeChannelCount(type == QHttpNetworkConnection::ConnectionTypeHTTP2
-                       || type == QHttpNetworkConnection::ConnectionTypeHTTP2Direct
-                       ? 1 : defaultHttpChannelCount)
-  , channelCount(defaultHttpChannelCount)
-#ifndef QT_NO_NETWORKPROXY
-  , networkProxy(QNetworkProxy::NoProxy)
-#endif
-  , preConnectRequests(0)
-  , connectionType(type)
+static int getPreferredActiveChannelCount(QHttpNetworkConnection::ConnectionType type,
+                                          int defaultValue)
 {
-    // We allocate all 6 channels even if it's HTTP/2 enabled connection:
-    // in case the protocol negotiation via NPN/ALPN fails, we will have
-    // normally working HTTP/1.1.
-    Q_ASSERT(channelCount >= activeChannelCount);
-    channels = new QHttpNetworkConnectionChannel[channelCount];
+    return (type == QHttpNetworkConnection::ConnectionTypeHTTP2
+            || type == QHttpNetworkConnection::ConnectionTypeHTTP2Direct)
+            ? 1
+            : defaultValue;
 }
 
-QHttpNetworkConnectionPrivate::QHttpNetworkConnectionPrivate(quint16 connectionCount, const QString &hostName,
-                                                             quint16 port, bool encrypt,
-                                                             QHttpNetworkConnection::ConnectionType type)
-: state(RunningState), networkLayerState(Unknown),
-  hostName(hostName), port(port), encrypt(encrypt), delayIpv4(true),
-  channelCount(connectionCount)
+QHttpNetworkConnectionPrivate::QHttpNetworkConnectionPrivate(
+        quint16 connectionCount, const QString &hostName, quint16 port, bool encrypt,
+        QHttpNetworkConnection::ConnectionType type)
+    : hostName(hostName),
+      port(port),
+      encrypt(encrypt),
+      activeChannelCount(getPreferredActiveChannelCount(type, connectionCount)),
+      channelCount(connectionCount),
+      channels(new QHttpNetworkConnectionChannel[channelCount]),
 #ifndef QT_NO_NETWORKPROXY
-  , networkProxy(QNetworkProxy::NoProxy)
+      networkProxy(QNetworkProxy::NoProxy),
 #endif
-  , preConnectRequests(0)
-  , connectionType(type)
+      connectionType(type)
 {
-    channels = new QHttpNetworkConnectionChannel[channelCount];
-
-    activeChannelCount = (type == QHttpNetworkConnection::ConnectionTypeHTTP2 ||
-                          type == QHttpNetworkConnection::ConnectionTypeHTTP2Direct)
-                       ? 1 : connectionCount;
     // We allocate all 6 channels even if it's an HTTP/2-enabled
     // connection: in case the protocol negotiation via NPN/ALPN fails,
     // we will have normally working HTTP/1.1.
@@ -315,7 +296,7 @@ void QHttpNetworkConnectionPrivate::prepareRequest(HttpMessagePair &messagePair)
         QByteArray host;
         if (add.setAddress(hostName)) {
             if (add.protocol() == QAbstractSocket::IPv6Protocol)
-                host = '[' + hostName.toLatin1() + ']'; //format the ipv6 in the standard way
+                host = (u'[' + hostName + u']').toLatin1(); //format the ipv6 in the standard way
             else
                 host = hostName.toLatin1();
 
@@ -411,7 +392,7 @@ bool QHttpNetworkConnectionPrivate::handleAuthenticateChallenge(QAbstractSocket
 
     resend = false;
     //create the response header to be used with QAuthenticatorPrivate.
-    QList<QPair<QByteArray, QByteArray> > fields = reply->header();
+    const auto headers = reply->header();
 
     // Check that any of the proposed authenticate methods are supported
     const QByteArray header = isProxy ? "proxy-authenticate" : "www-authenticate";
@@ -427,7 +408,7 @@ bool QHttpNetworkConnectionPrivate::handleAuthenticateChallenge(QAbstractSocket
         if (auth->isNull())
             auth->detach();
         QAuthenticatorPrivate *priv = QAuthenticatorPrivate::getPrivate(*auth);
-        priv->parseHttpResponse(fields, isProxy, reply->url().host());
+        priv->parseHttpResponse(headers, isProxy, reply->url().host());
         // Update method in case it changed
         if (priv->method == QAuthenticatorPrivate::None)
             return false;
@@ -523,12 +504,9 @@ QHttpNetworkConnectionPrivate::parseRedirectResponse(QHttpNetworkReply *reply)
         return {{}, QNetworkReply::NoError};
 
     QUrl redirectUrl;
-    const QList<QPair<QByteArray, QByteArray> > fields = reply->header();
-    for (const QNetworkReply::RawHeaderPair &header : fields) {
-        if (header.first.compare("location", Qt::CaseInsensitive) == 0) {
-            redirectUrl = QUrl::fromEncoded(header.second);
-            break;
-        }
+    const QHttpHeaders fields = reply->header();
+    if (const auto h = fields.values(QHttpHeaders::WellKnownHeader::Location); !h.empty()) {
+        redirectUrl = QUrl::fromEncoded(h.first());
     }
 
     // If the location url is invalid/empty, we return ProtocolUnknownError
@@ -990,19 +968,18 @@ void QHttpNetworkConnectionPrivate::removeReply(QHttpNetworkReply *reply)
                return;
             }
         }
-#ifndef QT_NO_SSL
         // is the reply inside the H2 pipeline of this channel already?
-        QMultiMap<int, HttpMessagePair>::iterator it = channels[i].h2RequestsToSend.begin();
-        QMultiMap<int, HttpMessagePair>::iterator end = channels[i].h2RequestsToSend.end();
-        for (; it != end; ++it) {
-            if (it.value().second == reply) {
-                channels[i].h2RequestsToSend.remove(it.key());
-
-                QMetaObject::invokeMethod(q, "_q_startNextRequest", Qt::QueuedConnection);
-                return;
-            }
+        const auto foundReply = [reply](const HttpMessagePair &pair) {
+            return pair.second == reply;
+        };
+        auto &seq = channels[i].h2RequestsToSend;
+        const auto end = seq.cend();
+        auto it = std::find_if(seq.cbegin(), end, foundReply);
+        if (it != end) {
+            seq.erase(it);
+            QMetaObject::invokeMethod(q, "_q_startNextRequest", Qt::QueuedConnection);
+            return;
         }
-#endif
     }
     // remove from the high priority queue
     if (!highPriorityQueue.isEmpty()) {
@@ -1046,6 +1023,11 @@ void QHttpNetworkConnectionPrivate::_q_startNextRequest()
     //resend the necessary ones.
     for (int i = 0; i < activeChannelCount; ++i) {
         if (channels[i].resendCurrent && (channels[i].state != QHttpNetworkConnectionChannel::ClosingState)) {
+            if (!channels[i].socket
+                || channels[i].socket->state() == QAbstractSocket::UnconnectedState) {
+                if (!channels[i].ensureConnection())
+                    continue;
+            }
             channels[i].resendCurrent = false;
 
             // if this is not possible, error will be emitted and connection terminated
@@ -1133,7 +1115,7 @@ void QHttpNetworkConnectionPrivate::_q_startNextRequest()
     if (neededOpenChannels <= 0)
         return;
 
-    QQueue<int> channelsToConnect;
+    QVarLengthArray<int> channelsToConnect;
 
     // use previously used channels first
     for (int i = 0; i < activeChannelCount && neededOpenChannels > 0; ++i) {
@@ -1149,7 +1131,7 @@ void QHttpNetworkConnectionPrivate::_q_startNextRequest()
 
         if (!channels[i].reply && !channels[i].isSocketBusy()
             && (channels[i].socket->state() == QAbstractSocket::UnconnectedState)) {
-            channelsToConnect.enqueue(i);
+            channelsToConnect.push_back(i);
             neededOpenChannels--;
         }
     }
@@ -1159,12 +1141,14 @@ void QHttpNetworkConnectionPrivate::_q_startNextRequest()
         if (channels[i].socket)
             continue;
 
-        channelsToConnect.enqueue(i);
+        channelsToConnect.push_back(i);
         neededOpenChannels--;
     }
 
-    while (!channelsToConnect.isEmpty()) {
-        const int channel = channelsToConnect.dequeue();
+    auto channelToConnectSpan = QSpan{channelsToConnect};
+    while (!channelToConnectSpan.isEmpty()) {
+        const int channel = channelToConnectSpan.front();
+        channelToConnectSpan = channelToConnectSpan.sliced(1);
 
         if (networkLayerState == IPv4)
             channels[channel].networkLayerPreference = QAbstractSocket::IPv4Protocol;
@@ -1344,18 +1328,6 @@ void QHttpNetworkConnectionPrivate::_q_connectDelayedChannel()
         channels[1].ensureConnection();
 }
 
-QHttpNetworkConnection::QHttpNetworkConnection(const QString &hostName, quint16 port, bool encrypt,
-                                               QHttpNetworkConnection::ConnectionType connectionType, QObject *parent)
-    : QObject(*(new QHttpNetworkConnectionPrivate(hostName, port, encrypt , connectionType)), parent)
-{
-    Q_D(QHttpNetworkConnection);
-    d->init();
-    if (QNetworkConnectionMonitor::isEnabled()) {
-        connect(&d->connectionMonitor, &QNetworkConnectionMonitor::reachabilityChanged,
-                this, &QHttpNetworkConnection::onlineStateChanged, Qt::QueuedConnection);
-    }
-}
-
 QHttpNetworkConnection::QHttpNetworkConnection(quint16 connectionCount, const QString &hostName,
                                                quint16 port, bool encrypt, QObject *parent,
                                                QHttpNetworkConnection::ConnectionType connectionType)
@@ -1443,9 +1415,9 @@ QNetworkProxy QHttpNetworkConnection::transparentProxy() const
 }
 #endif
 
-QHttpNetworkConnection::ConnectionType QHttpNetworkConnection::connectionType()
+QHttpNetworkConnection::ConnectionType QHttpNetworkConnection::connectionType() const
 {
-    Q_D(QHttpNetworkConnection);
+    Q_D(const QHttpNetworkConnection);
     return d->connectionType;
 }
 
@@ -1480,9 +1452,9 @@ void QHttpNetworkConnection::setSslConfiguration(const QSslConfiguration &config
         d->channels[i].setSslConfiguration(config);
 }
 
-std::shared_ptr<QSslContext> QHttpNetworkConnection::sslContext()
+std::shared_ptr<QSslContext> QHttpNetworkConnection::sslContext() const
 {
-    Q_D(QHttpNetworkConnection);
+    Q_D(const QHttpNetworkConnection);
     return d->sslContext;
 }
 
