@@ -12,7 +12,7 @@
 #include <qdir.h>
 #include <qdatastream.h>
 #include <qdatetime.h>
-#include <qdiriterator.h>
+#include <qdirlisting.h>
 #include <qurl.h>
 #include <qcryptographichash.h>
 #include <qdebug.h>
@@ -172,11 +172,7 @@ QIODevice *QNetworkDiskCache::prepare(const QNetworkCacheMetaData &metaData)
         device = &(cacheItem->data);
     } else {
         QString fileName = d->cacheFileName(cacheItem->metaData.url());
-        QT_TRY {
-            cacheItem->file = new QSaveFile(fileName, &cacheItem->data);
-        } QT_CATCH(...) {
-            cacheItem->file = nullptr;
-        }
+        cacheItem->file = new(std::nothrow) QSaveFile(fileName, &cacheItem->data);
         if (!cacheItem->file || !cacheItem->file->open(QFileDevice::WriteOnly)) {
             qWarning("QNetworkDiskCache::prepare() unable to open temporary file");
             cacheItem.reset();
@@ -482,7 +478,6 @@ qint64 QNetworkDiskCache::expire()
     d->lastItem.reset();
 
     const QDir::Filters filters = QDir::AllDirs | QDir:: Files | QDir::NoDotAndDotDot;
-    QDirIterator it(cacheDirectory(), filters, QDirIterator::Subdirectories);
 
     struct CacheItem
     {
@@ -492,11 +487,12 @@ qint64 QNetworkDiskCache::expire()
     };
     std::vector<CacheItem> cacheItems;
     qint64 totalSize = 0;
-    while (it.hasNext()) {
-        QFileInfo info = it.nextFileInfo();
-        if (!info.fileName().endsWith(CACHE_POSTFIX))
+    using F = QDirListing::IteratorFlag;
+    for (const auto &dirEntry : QDirListing(cacheDirectory(), filters, F::Recursive)) {
+        if (!dirEntry.fileName().endsWith(CACHE_POSTFIX))
             continue;
 
+        const QFileInfo &info = dirEntry.fileInfo();
         QDateTime fileTime = info.birthTime(QTimeZone::UTC);
         if (!fileTime.isValid())
             fileTime = info.metadataChangeTime(QTimeZone::UTC);
