@@ -34,6 +34,7 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 using namespace QtMiscUtils;
+using namespace std::chrono_literals;
 
 class QNetworkProxy;
 
@@ -510,7 +511,7 @@ bool QNetworkReplyHttpImplPrivate::loadFromCacheIfAllowed(QHttpNetworkRequest &h
 
     it = cacheHeaders.findRawHeader("content-length");
     if (it != cacheHeaders.rawHeaders.constEnd()) {
-        QIODevice *data = nc->data(httpRequest.url());
+        std::unique_ptr<QIODevice> data(nc->data(httpRequest.url()));
         if (!data || data->size() < it->second.toLongLong())
             return false; // The data is smaller than the content-length specified
     }
@@ -699,8 +700,13 @@ void QNetworkReplyHttpImplPrivate::postRequest(const QNetworkRequest &newHttpReq
     switch (operation) {
     case QNetworkAccessManager::GetOperation:
         httpRequest.setOperation(QHttpNetworkRequest::Get);
-        if (loadFromCacheIfAllowed(httpRequest))
+        // If the request has a body, createUploadByteDevice() and don't use caching
+        if (outgoingData) {
+            invalidateCache();
+            createUploadByteDevice();
+        } else if (loadFromCacheIfAllowed(httpRequest)) {
             return; // no need to send the request! :)
+        }
         break;
 
     case QNetworkAccessManager::HeadOperation:
@@ -1239,13 +1245,18 @@ void QNetworkReplyHttpImplPrivate::onRedirected(const QUrl &redirectUrl, int htt
         return;
     }
 
+    // If the original operation was a GET with a body and the status code is either
+    // 307 or 308 then keep the message body
+    const bool getOperationKeepsBody = (operation == QNetworkAccessManager::GetOperation)
+                          && (httpStatus == 307 || httpStatus == 308);
+
     redirectRequest = createRedirectRequest(originalRequest, url, maxRedirectsRemaining);
     operation = getRedirectOperation(operation, httpStatus);
 
     // Clear stale headers, the relevant ones get set again later
     httpRequest.clearHeaders();
-    if (operation == QNetworkAccessManager::GetOperation
-        || operation == QNetworkAccessManager::HeadOperation) {
+    if ((operation == QNetworkAccessManager::GetOperation
+        || operation == QNetworkAccessManager::HeadOperation) && !getOperationKeepsBody) {
         // possibly changed from not-GET/HEAD to GET/HEAD, make sure to get rid of upload device
         uploadByteDevice.reset();
         uploadByteDevicePosition = 0;
@@ -1292,6 +1303,8 @@ void QNetworkReplyHttpImplPrivate::followRedirect()
             q, [this]() { postRequest(redirectRequest); }, Qt::QueuedConnection);
 }
 
+static constexpr QLatin1StringView locationHeader() noexcept { return "location"_L1; }
+
 void QNetworkReplyHttpImplPrivate::checkForRedirect(const int statusCode)
 {
     Q_Q(QNetworkReplyHttpImpl);
@@ -1304,7 +1317,7 @@ void QNetworkReplyHttpImplPrivate::checkForRedirect(const int statusCode)
         // What do we do about the caching of the HTML note?
         // The response to a 303 MUST NOT be cached, while the response to
         // all of the others is cacheable if the headers indicate it to be
-        QByteArray header = q->rawHeader("location"_ba);
+        QByteArray header = q->rawHeader(locationHeader());
         QUrl url = QUrl(QString::fromUtf8(header));
         if (!url.isValid())
             url = QUrl(QLatin1StringView(header));
@@ -1312,7 +1325,7 @@ void QNetworkReplyHttpImplPrivate::checkForRedirect(const int statusCode)
     }
 }
 
-void QNetworkReplyHttpImplPrivate::replyDownloadMetaData(const QList<QPair<QByteArray,QByteArray> > &hm,
+void QNetworkReplyHttpImplPrivate::replyDownloadMetaData(const QHttpHeaders &hm,
                                                          int sc, const QString &rp, bool pu,
                                                          QSharedPointer<char> db,
                                                          qint64 contentLength,
@@ -1348,21 +1361,22 @@ void QNetworkReplyHttpImplPrivate::replyDownloadMetaData(const QList<QPair<QByte
     // A user having manually defined which encodings they accept is, for
     // somwehat unknown (presumed legacy compatibility) reasons treated as
     // disabling our decompression:
-    const bool autoDecompress = request.rawHeader("accept-encoding"_ba).isEmpty();
+    const bool autoDecompress = request.rawHeader("accept-encoding").isEmpty();
     const bool shouldDecompress = isCompressed && autoDecompress;
     // reconstruct the HTTP header
-    for (const auto &[key, originValue] : hm) {
+    for (qsizetype i = 0; i < hm.size(); ++i) {
+        const auto key = hm.nameAt(i);
+        const auto originValue = hm.valueAt(i);
+
         QByteArray value = q->rawHeader(key);
 
         // Reset any previous "location" header set in the reply. In case of
         // redirects, we don't want to 'append' multiple location header values,
         // rather we keep only the latest one
-        if (key.toLower() == "location")
+        if (key == locationHeader())
             value.clear();
 
-        if (shouldDecompress && !decompressHelper.isValid()
-            && key.compare("content-encoding", Qt::CaseInsensitive) == 0) {
-
+        if (shouldDecompress && !decompressHelper.isValid() && key == "content-encoding"_L1) {
             if (!synchronous) // with synchronous all the data is expected to be handled at once
                 decompressHelper.setCountingBytesEnabled(true);
 
@@ -1379,13 +1393,13 @@ void QNetworkReplyHttpImplPrivate::replyDownloadMetaData(const QList<QPair<QByte
         if (!value.isEmpty()) {
             // Why are we appending values for headers which are already
             // present?
-            if (key.compare("set-cookie", Qt::CaseInsensitive) == 0)
+            if (key == "set-cookie"_L1)
                 value += '\n';
             else
                 value += ", ";
         }
         value += originValue;
-        q->setRawHeader(key, value);
+        q->setRawHeader({key.data(), key.size()}, value);
     }
 
     q->setAttribute(QNetworkRequest::HttpStatusCodeAttribute, statusCode);
@@ -1652,7 +1666,7 @@ bool QNetworkReplyHttpImplPrivate::sendCacheContents(const QNetworkCacheMetaData
     QUrl redirectUrl;
     for ( ; it != end; ++it) {
         if (httpRequest.isFollowRedirects() &&
-            !it->first.compare("location", Qt::CaseInsensitive))
+            !it->first.compare(locationHeader(), Qt::CaseInsensitive))
             redirectUrl = QUrl::fromEncoded(it->second);
         setRawHeader(it->first, it->second);
     }
@@ -1849,7 +1863,7 @@ bool QNetworkReplyHttpImplPrivate::canResume() const
         return false;
 
     // Can only resume if server/resource supports Range header.
-    const auto acceptRangesheaderName = "Accept-Ranges"_ba;
+    constexpr auto acceptRangesheaderName = QByteArrayView("Accept-Ranges");
     if (!q->hasRawHeader(acceptRangesheaderName) || q->rawHeader(acceptRangesheaderName) == "none")
         return false;
 
@@ -1875,7 +1889,9 @@ void QNetworkReplyHttpImplPrivate::setResumeOffset(quint64 offset)
 
 void QNetworkReplyHttpImplPrivate::_q_startOperation()
 {
-    if (state == Working) // ensure this function is only being called once
+    // Ensure this function is only being called once, and not at all if we were
+    // cancelled
+    if (state >= Working)
         return;
 
     state = Working;
@@ -2035,9 +2051,9 @@ void QNetworkReplyHttpImplPrivate::setupTransferTimeout()
                        Qt::QueuedConnection);
     }
     transferTimeout->stop();
-    if (request.transferTimeout()) {
+    if (request.transferTimeoutAsDuration() > 0ms) {
         transferTimeout->setSingleShot(true);
-        transferTimeout->setInterval(request.transferTimeout());
+        transferTimeout->setInterval(request.transferTimeoutAsDuration());
         QMetaObject::invokeMethod(transferTimeout, "start",
                                   Qt::QueuedConnection);
 
@@ -2143,7 +2159,9 @@ void QNetworkReplyHttpImplPrivate::error(QNetworkReplyImpl::NetworkError code, c
     Q_Q(QNetworkReplyHttpImpl);
     // Can't set and emit multiple errors.
     if (errorCode != QNetworkReply::NoError) {
-        qWarning("QNetworkReplyImplPrivate::error: Internal problem, this method must only be called once.");
+        // But somewhat unavoidable if we have cancelled the request:
+        if (errorCode != QNetworkReply::OperationCanceledError)
+            qWarning("QNetworkReplyImplPrivate::error: Internal problem, this method must only be called once.");
         return;
     }
 
