@@ -22,6 +22,7 @@
 #include <QtGui/qpainter.h>
 #include <QtGui/qtextformat.h>
 #include <QtGui/private/qcoretextfontdatabase_p.h>
+#include <QtGui/private/qappleiconengine_p.h>
 #include <QtGui/private/qfontengine_coretext_p.h>
 #include <QtGui/private/qabstractfileiconengine_p.h>
 #include <qpa/qplatformdialoghelper.h>
@@ -409,19 +410,8 @@ public:
                                   QPlatformTheme::IconOptions opts) :
         QAbstractFileIconEngine(info, opts) {}
 
-    static QList<QSize> availableIconSizes()
-    {
-        const qreal devicePixelRatio = qGuiApp->devicePixelRatio();
-        const int sizes[] = {
-            qRound(16 * devicePixelRatio), qRound(32 * devicePixelRatio),
-            qRound(64 * devicePixelRatio), qRound(128 * devicePixelRatio),
-            qRound(256 * devicePixelRatio)
-        };
-        return QAbstractFileIconEngine::toSizeList(sizes, sizes + sizeof(sizes) / sizeof(sizes[0]));
-    }
-
     QList<QSize> availableSizes(QIcon::Mode = QIcon::Normal, QIcon::State = QIcon::Off) override
-    { return QCocoaFileIconEngine::availableIconSizes(); }
+    { return QAppleIconEngine::availableIconSizes(); }
 
 protected:
     QPixmap filePixmap(const QSize &size, QIcon::Mode, QIcon::State) override
@@ -440,6 +430,11 @@ QIcon QCocoaTheme::fileIcon(const QFileInfo &fileInfo, QPlatformTheme::IconOptio
     return QIcon(new QCocoaFileIconEngine(fileInfo, iconOptions));
 }
 
+QIconEngine *QCocoaTheme::createIconEngine(const QString &iconName) const
+{
+    return new QAppleIconEngine(iconName);
+}
+
 QVariant QCocoaTheme::themeHint(ThemeHint hint) const
 {
     switch (hint) {
@@ -453,7 +448,7 @@ QVariant QCocoaTheme::themeHint(ThemeHint hint) const
         return QVariant([[NSApplication sharedApplication] isFullKeyboardAccessEnabled] ?
                     int(Qt::TabFocusAllControls) : int(Qt::TabFocusTextControls | Qt::TabFocusListControls));
     case IconPixmapSizes:
-        return QVariant::fromValue(QCocoaFileIconEngine::availableIconSizes());
+        return QVariant::fromValue(QAppleIconEngine::availableIconSizes());
     case QPlatformTheme::PasswordMaskCharacter:
         return QVariant(QChar(0x2022));
     case QPlatformTheme::UiEffects:
@@ -510,12 +505,16 @@ QKeySequence QCocoaTheme::standardButtonShortcut(int button) const
 
 QPlatformMenuItem *QCocoaTheme::createPlatformMenuItem() const
 {
-    return new QCocoaMenuItem();
+    auto *menuItem = new QCocoaMenuItem();
+    qCDebug(lcQpaMenus) << "Created" << menuItem;
+    return menuItem;
 }
 
 QPlatformMenu *QCocoaTheme::createPlatformMenu() const
 {
-    return new QCocoaMenu();
+    auto *menu = new QCocoaMenu();
+    qCDebug(lcQpaMenus) << "Created" << menu;
+    return menu;
 }
 
 QPlatformMenuBar *QCocoaTheme::createPlatformMenuBar() const
@@ -528,7 +527,9 @@ QPlatformMenuBar *QCocoaTheme::createPlatformMenuBar() const
                 SLOT(onAppFocusWindowChanged(QWindow*)));
     }
 
-    return new QCocoaMenuBar();
+    auto *menuBar = new QCocoaMenuBar();
+    qCDebug(lcQpaMenus) << "Created" << menuBar;
+    return menuBar;
 }
 
 #ifndef QT_NO_SHORTCUT
