@@ -21,7 +21,7 @@
 #import <QuartzCore/CAEAGLLayer.h>
 #endif
 
-#ifdef Q_OS_IOS
+#if QT_CONFIG(metal)
 #import <QuartzCore/CAMetalLayer.h>
 #endif
 
@@ -29,6 +29,11 @@
 
 QT_BEGIN_NAMESPACE
 
+enum {
+    defaultWindowWidth = 160,
+    defaultWindowHeight = 160
+};
+
 QIOSWindow::QIOSWindow(QWindow *window, WId nativeHandle)
     : QPlatformWindow(window)
     , m_windowLevel(0)
@@ -37,7 +42,7 @@ QIOSWindow::QIOSWindow(QWindow *window, WId nativeHandle)
         m_view = reinterpret_cast<UIView *>(nativeHandle);
         [m_view retain];
     } else {
-#ifdef Q_OS_IOS
+#if QT_CONFIG(metal)
         if (window->surfaceType() == QSurface::RasterSurface)
             window->setSurfaceType(QSurface::MetalSurface);
 
@@ -54,12 +59,9 @@ QIOSWindow::QIOSWindow(QWindow *window, WId nativeHandle)
 
     if (!isForeignWindow()) {
         // Resolve default window geometry in case it was not set before creating the
-        // platform window. This picks up eg. minimum-size if set, and defaults to
-        // the "maxmized" geometry (even though we're not in that window state).
-        // FIXME: Detect if we apply a maximized geometry and send a window state
-        // change event in that case.
+        // platform window. This picks up eg. minimum-size if set.
         m_normalGeometry = initialGeometry(window, QPlatformWindow::geometry(),
-            screen()->availableGeometry().width(), screen()->availableGeometry().height());
+            defaultWindowWidth, defaultWindowHeight);
 
         setWindowState(window->windowStates());
         setOpacity(window->opacity());
@@ -225,7 +227,7 @@ void QIOSWindow::applyGeometry(const QRect &rect)
 
 QMargins QIOSWindow::safeAreaMargins() const
 {
-    UIEdgeInsets safeAreaInsets = m_view.qt_safeAreaInsets;
+    UIEdgeInsets safeAreaInsets = m_view.safeAreaInsets;
     return QMargins(safeAreaInsets.left, safeAreaInsets.top,
         safeAreaInsets.right, safeAreaInsets.bottom);
 }
@@ -249,22 +251,45 @@ void QIOSWindow::setWindowState(Qt::WindowStates state)
     if (state & Qt::WindowMinimized) {
         applyGeometry(QRect());
     } else if (state & (Qt::WindowFullScreen | Qt::WindowMaximized)) {
-        // When an application is in split-view mode, the UIScreen still has the
-        // same geometry, but the UIWindow is resized to the area reserved for the
-        // application. We use this to constrain the geometry used when applying the
-        // fullscreen or maximized window states. Note that we do not do this
-        // in applyGeometry(), as we don't want to artificially limit window
-        // placement "outside" of the screen bounds if that's what the user wants.
-
         QRect uiWindowBounds = QRectF::fromCGRect(m_view.window.bounds).toRect();
-        QRect fullscreenGeometry = screen()->geometry().intersected(uiWindowBounds);
-        QRect maximizedGeometry = window()->flags() & Qt::MaximizeUsingFullscreenGeometryHint ?
-            fullscreenGeometry : screen()->availableGeometry().intersected(uiWindowBounds);
+        if (NSProcessInfo.processInfo.iOSAppOnMac) {
+            // iOS apps running as "Designed for iPad" on macOS do not match
+            // our current window management implementation where a single
+            // UIWindow is tied to a single screen. And even if we're on the
+            // right screen, the UIScreen does not account for the 77% scale
+            // of the UIUserInterfaceIdiomPad environment, so we can't use
+            // it to clamp the window geometry. Instead just use the UIWindow
+            // directly, which represents our "screen".
+            applyGeometry(uiWindowBounds);
+        } else if (isRunningOnVisionOS()) {
+            // On visionOS there is no concept of a screen, and hence no concept of
+            // screen-relative system UI that we should keep top level windows away
+            // from, so don't apply the UIWindow safe area insets to the screen.
+            applyGeometry(uiWindowBounds);
+        } else {
+            QRect fullscreenGeometry = screen()->geometry();
+            QRect maximizedGeometry = fullscreenGeometry;
 
-        if (state & Qt::WindowFullScreen)
-            applyGeometry(fullscreenGeometry);
-        else
-            applyGeometry(maximizedGeometry);
+#if !defined(Q_OS_VISIONOS)
+            if (!(window()->flags() & Qt::MaximizeUsingFullscreenGeometryHint)) {
+                // If the safe area margins reflect the screen's outer edges,
+                // then reduce the maximized geometry accordingly. Otherwise
+                // leave it as is, and assume the client will take the safe
+                // are margins into account explicitly.
+                UIScreen *uiScreen = m_view.window.windowScene.screen;
+                UIEdgeInsets safeAreaInsets = m_view.window.safeAreaInsets;
+                if (m_view.window.bounds.size.width == uiScreen.bounds.size.width)
+                    maximizedGeometry.adjust(safeAreaInsets.left, 0, -safeAreaInsets.right, 0);
+                if (m_view.window.bounds.size.height == uiScreen.bounds.size.height)
+                    maximizedGeometry.adjust(0, safeAreaInsets.top, 0, -safeAreaInsets.bottom);
+            }
+#endif
+
+            if (state & Qt::WindowFullScreen)
+                applyGeometry(fullscreenGeometry.intersected(uiWindowBounds));
+            else
+                applyGeometry(maximizedGeometry.intersected(uiWindowBounds));
+        }
     } else {
         applyGeometry(m_normalGeometry);
     }
@@ -292,7 +317,6 @@ void QIOSWindow::requestActivateWindow()
     if (blockedByModal())
         return;
 
-    Q_ASSERT(m_view.window);
     [m_view.window makeKeyWindow];
     [m_view becomeFirstResponder];
 
@@ -358,16 +382,6 @@ void QIOSWindow::updateWindowLevel()
         m_windowLevel = qMax(transientParentWindow->m_windowLevel, m_windowLevel);
 }
 
-void QIOSWindow::handleContentOrientationChange(Qt::ScreenOrientation orientation)
-{
-    // Update the QWindow representation straight away, so that
-    // we can update the statusbar orientation based on the new
-    // content orientation.
-    qt_window_private(window())->contentOrientation = orientation;
-
-    [m_view.qtViewController updateProperties];
-}
-
 void QIOSWindow::applicationStateChanged(Qt::ApplicationState)
 {
     if (isForeignWindow())
