@@ -80,31 +80,6 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
-/*!
-    \class QWindowsIntegration
-    \brief QPlatformIntegration implementation for Windows.
-    \internal
-
-    \section1 Programming Considerations
-
-    The platform plugin should run on Desktop Windows from Windows XP onwards
-    and Windows Embedded.
-
-    It should compile with:
-    \list
-    \li Microsoft Visual Studio 2013 or later (using the Microsoft Windows SDK,
-        (\c Q_CC_MSVC).
-    \li Stock \l{http://mingw.org/}{MinGW} (\c Q_CC_MINGW).
-        This version ships with headers that are missing a lot of WinAPI.
-    \li MinGW distributions using GCC 4.7 or higher and a recent MinGW-w64 runtime API,
-        such as \l{http://tdm-gcc.tdragon.net/}{TDM-GCC}, or
-        \l{http://mingwbuilds.sourceforge.net/}{MinGW-builds}
-        (\c Q_CC_MINGW and \c __MINGW64_VERSION_MAJOR indicating the version).
-        MinGW-w64 provides more complete headers (compared to stock MinGW from mingw.org),
-        including a considerable part of the Windows SDK.
-    \endlist
-*/
-
 struct QWindowsIntegrationPrivate
 {
     Q_DISABLE_COPY_MOVE(QWindowsIntegrationPrivate)
@@ -167,8 +142,8 @@ static inline unsigned parseOptions(const QStringList &paramList,
     unsigned options = 0;
     for (const QString &param : paramList) {
         if (param.startsWith(u"fontengine=")) {
-            if (param.endsWith(u"directwrite")) {
-                options |= QWindowsIntegration::FontDatabaseDirectWrite;
+            if (param.endsWith(u"gdi")) {
+                options |= QWindowsIntegration::FontDatabaseGDI;
             } else if (param.endsWith(u"freetype")) {
                 options |= QWindowsIntegration::FontDatabaseFreeType;
             } else if (param.endsWith(u"native")) {
@@ -281,9 +256,9 @@ QWindowsIntegration::~QWindowsIntegration()
 
 void QWindowsIntegration::initialize()
 {
-    QString icStr = QPlatformInputContextFactory::requested();
-    icStr.isNull() ? d->m_inputContext.reset(new QWindowsInputContext)
-                   : d->m_inputContext.reset(QPlatformInputContextFactory::create(icStr));
+    auto icStrs = QPlatformInputContextFactory::requested();
+    icStrs.isEmpty() ? d->m_inputContext.reset(new QWindowsInputContext)
+                     : d->m_inputContext.reset(QPlatformInputContextFactory::create(icStrs));
 }
 
 bool QWindowsIntegration::hasCapability(QPlatformIntegration::Capability cap) const
@@ -311,6 +286,8 @@ bool QWindowsIntegration::hasCapability(QPlatformIntegration::Capability cap) co
         return true;
     case SwitchableWidgetComposition:
         return false; // QTBUG-68329 QTBUG-53515 QTBUG-54734
+    case BackingStoreStaticContents:
+        return true;
     default:
         return QPlatformIntegration::hasCapability(cap);
     }
@@ -507,17 +484,17 @@ QWindowsStaticOpenGLContext *QWindowsIntegration::staticOpenGLContext()
 QPlatformFontDatabase *QWindowsIntegration::fontDatabase() const
 {
     if (!d->m_fontDatabase) {
-#if QT_CONFIG(directwrite3)
-        if (d->m_options & QWindowsIntegration::FontDatabaseDirectWrite)
-            d->m_fontDatabase = new QWindowsDirectWriteFontDatabase;
-        else
-#endif
 #ifndef QT_NO_FREETYPE
         if (d->m_options & QWindowsIntegration::FontDatabaseFreeType)
             d->m_fontDatabase = new QWindowsFontDatabaseFT;
         else
 #endif // QT_NO_FREETYPE
-        d->m_fontDatabase = new QWindowsFontDatabase();
+#if QT_CONFIG(directwrite3)
+        if (!(d->m_options & (QWindowsIntegration::FontDatabaseGDI | QWindowsIntegration::DontUseDirectWriteFonts)))
+            d->m_fontDatabase = new QWindowsDirectWriteFontDatabase;
+        else
+#endif
+            d->m_fontDatabase = new QWindowsFontDatabase;
     }
     return d->m_fontDatabase;
 }
@@ -565,14 +542,9 @@ QVariant QWindowsIntegration::styleHint(QPlatformIntegration::StyleHint hint) co
     return QPlatformIntegration::styleHint(hint);
 }
 
-Qt::KeyboardModifiers QWindowsIntegration::queryKeyboardModifiers() const
+QPlatformKeyMapper *QWindowsIntegration::keyMapper() const
 {
-    return QWindowsKeyMapper::queryKeyboardModifiers();
-}
-
-QList<int> QWindowsIntegration::possibleKeys(const QKeyEvent *e) const
-{
-    return d->m_context.possibleKeys(e);
+    return d->m_context.keyMapper();
 }
 
 #if QT_CONFIG(clipboard)
@@ -785,7 +757,8 @@ void QWindowsIntegration::updateApplicationBadge()
     // to a task bar button being created for the fist time or after
     // Explorer had crashed and re-started. In any case, re-apply the
     // badge so that everything is up to date.
-    setApplicationBadge(m_applicationBadgeNumber);
+    if (m_applicationBadgeNumber)
+        setApplicationBadge(m_applicationBadgeNumber);
 }
 
 #if QT_CONFIG(vulkan)
