@@ -1,7 +1,8 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
+#include <QtTest/private/qcomparisontesthelper_p.h>
 #include <QStandardPaths>
 #include <QScopeGuard>
 
@@ -172,6 +173,7 @@ private slots:
 
     void systemFiles();
 
+    void compareCompiles();
     void compare_data();
     void compare();
 
@@ -351,7 +353,7 @@ void tst_QFileInfo::isDir_data()
     QFile::remove("brokenlink.lnk");
     QFile::remove("dummyfile");
     QFile file3("dummyfile");
-    file3.open(QIODevice::WriteOnly);
+    QVERIFY(file3.open(QIODevice::WriteOnly));
     if (file3.link("brokenlink.lnk")) {
         file3.remove();
         QFileInfo info3("brokenlink.lnk");
@@ -656,7 +658,7 @@ void tst_QFileInfo::canonicalFilePath()
         QFile file(QDir::currentPath());
         if (file.link(link)) {
             QFile tempfile("tempfile.txt");
-            tempfile.open(QIODevice::ReadWrite);
+            QVERIFY(tempfile.open(QIODevice::ReadWrite));
             tempfile.write("This file is generated by the QFileInfo autotest.");
             QVERIFY(tempfile.flush());
             tempfile.close();
@@ -993,6 +995,11 @@ void tst_QFileInfo::systemFiles()
     QVERIFY(fi.birthTime() <= fi.lastModified());
 }
 
+void tst_QFileInfo::compareCompiles()
+{
+    QTestPrivate::testEqualityOperatorsCompile<QFileInfo>();
+}
+
 void tst_QFileInfo::compare_data()
 {
     QTest::addColumn<QString>("file1");
@@ -1037,7 +1044,7 @@ void tst_QFileInfo::compare()
     QFETCH(QString, file2);
     QFETCH(bool, same);
     QFileInfo fi1(file1), fi2(file2);
-    QCOMPARE(fi1 == fi2, same);
+    QT_TEST_EQUALITY_OPS(fi1, fi2, same);
 }
 
 void tst_QFileInfo::consistent_data()
@@ -1233,7 +1240,7 @@ void tst_QFileInfo::fakeFileTimes()
     QFETCH(QDateTime, when);
 
     QFile file("faketimefile.txt");
-    file.open(QIODevice::WriteOnly);
+    QVERIFY(file.open(QIODevice::WriteOnly));
     file.write("\n", 1);
     file.close();
 
@@ -1242,7 +1249,7 @@ void tst_QFileInfo::fakeFileTimes()
       the file is open at the time.  Of course, when writing, close() changes
       modification time, so need to re-open for read in order to setFileTime().
      */
-    file.open(QIODevice::ReadOnly);
+    QVERIFY(file.open(QIODevice::ReadOnly));
     bool ok = file.setFileTime(when, QFileDevice::FileModificationTime);
     file.close();
 
@@ -1264,7 +1271,7 @@ void tst_QFileInfo::isSymLink_data()
     QVERIFY(file1.link("link.lnk"));
 
     QFile file2("dummyfile");
-    file2.open(QIODevice::WriteOnly);
+    QVERIFY(file2.open(QIODevice::WriteOnly));
     QVERIFY(file2.link("brokenlink.lnk"));
     file2.remove();
 
@@ -1450,7 +1457,7 @@ void tst_QFileInfo::link_data()
 
     QFile file1(m_sourceFile);
     QFile file2("dummyfile");
-    file2.open(QIODevice::WriteOnly);
+    QVERIFY(file2.open(QIODevice::WriteOnly));
 
     QTest::newRow("existent file") << m_sourceFile << false << false << "";
 #if defined(Q_OS_WIN)
@@ -1887,7 +1894,7 @@ void tst_QFileInfo::brokenShortcut()
 void tst_QFileInfo::isWritable()
 {
     QFile tempfile("tempfile.txt");
-    tempfile.open(QIODevice::WriteOnly);
+    QVERIFY(tempfile.open(QIODevice::WriteOnly));
     tempfile.write("This file is generated by the QFileInfo autotest.");
     tempfile.close();
 
