@@ -1,6 +1,6 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // Copyright (C) 2017 Intel Corporation.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <qcoreapplication.h>
@@ -75,6 +75,7 @@ private slots:
     void QTBUG_4796_data();
     void QTBUG_4796();
     void guaranteeUnique();
+    void stdfilesystem();
 private:
     QTemporaryDir m_temporaryDir;
     QString m_previousCurrent;
@@ -253,7 +254,7 @@ void tst_QTemporaryFile::fileName()
     QString absoluteTempPath = QDir(tempPath).absolutePath();
     QTemporaryFile file;
     file.setAutoRemove(true);
-    file.open();
+    QVERIFY(file.open());
     QString fileName = file.fileName();
     QVERIFY2(fileName.contains("/tst_qtemporaryfile."), qPrintable(fileName));
     QVERIFY(QFile::exists(fileName));
@@ -425,7 +426,7 @@ void tst_QTemporaryFile::io()
 
     file.reset();
     QFile compare(file.fileName());
-    compare.open(QIODevice::ReadOnly);
+    QVERIFY(compare.open(QIODevice::ReadOnly));
     QCOMPARE(compare.readAll() , data);
     QCOMPARE(compare.fileTime(QFile::FileModificationTime), mtime);
 }
@@ -459,7 +460,7 @@ void tst_QTemporaryFile::removeAndReOpen()
     QString fileName;
     {
         QTemporaryFile file;
-        file.open();
+        QVERIFY(file.open());
         fileName = file.fileName();     // materializes any unnamed file
         QVERIFY(QFile::exists(fileName));
 
@@ -479,7 +480,7 @@ void tst_QTemporaryFile::removeAndReOpen()
 void tst_QTemporaryFile::removeUnnamed()
 {
     QTemporaryFile file;
-    file.open();
+    QVERIFY(file.open());
 
     // we did not call fileName(), so the file name may not have a name
     QVERIFY(file.remove());
@@ -629,7 +630,7 @@ void tst_QTemporaryFile::renameFdLeak()
 
 void tst_QTemporaryFile::moveToTrash()
 {
-#if defined(Q_OS_ANDROID) || defined(Q_OS_WEBOS)
+#if defined(Q_OS_ANDROID) || defined(Q_OS_WEBOS) || defined(Q_OS_VXWORKS)
     QSKIP("This platform doesn't implement a trash bin");
 #endif
 #ifdef Q_OS_WIN
@@ -869,7 +870,7 @@ void tst_QTemporaryFile::createNativeFile()
 
     QFile f(filePath);
     if (currentPos != -1) {
-        f.open(QIODevice::ReadOnly);
+        QVERIFY(f.open(QIODevice::ReadOnly));
         f.seek(currentPos);
     }
     QTemporaryFile *tempFile = QTemporaryFile::createNativeFile(f);
@@ -1014,7 +1015,7 @@ void tst_QTemporaryFile::guaranteeUnique()
     // First pass. See which filename QTemporaryFile will try first.
     {
         QTemporaryFile tmpFile("testFile1.XXXXXX");
-        tmpFile.open();
+        QVERIFY(tmpFile.open());
         takenFileName = tmpFile.fileName();
         QVERIFY(QFile::exists(takenFileName));
     }
@@ -1036,5 +1037,49 @@ void tst_QTemporaryFile::guaranteeUnique()
     QVERIFY(dir.rmdir(takenFileName));
 }
 
+void tst_QTemporaryFile::stdfilesystem()
+{
+#if !QT_CONFIG(cxx17_filesystem)
+    QSKIP("std::filesystem not available");
+#else
+    // ctor
+    {
+        std::filesystem::path testFile("testFile1.XXXXXX");
+        QTemporaryFile file(testFile);
+        QCOMPARE(file.fileTemplate(), QtPrivate::fromFilesystemPath(testFile));
+    }
+    // rename
+    {
+        QTemporaryFile file("testFile1.XXXXXX");
+        QVERIFY(file.open());
+        QByteArray payload = "abc123 I am a string";
+        file.write(payload);
+        QVERIFY(file.rename(std::filesystem::path("./test")));
+        file.close();
+
+        QFile f(u"./test"_s);
+        QVERIFY(f.exists());
+        QVERIFY(f.open(QFile::ReadOnly));
+        QCOMPARE(f.readAll(), payload);
+    }
+    // createNativeFile
+    {
+        std::filesystem::path resource(":/resources/test.txt");
+        std::unique_ptr<QTemporaryFile> tmp(QTemporaryFile::createNativeFile(resource));
+        QVERIFY(tmp);
+        QFile file(resource);
+        QVERIFY(file.open(QFile::ReadOnly));
+        QCOMPARE(tmp->readAll(), file.readAll());
+    }
+    // setFileTemplate
+    {
+        QTemporaryFile file;
+        std::filesystem::path testFile("testFile1.XXXXXX");
+        file.setFileTemplate(testFile);
+        QCOMPARE(file.fileTemplate(), QtPrivate::fromFilesystemPath(testFile));
+    }
+#endif
+}
+
 QTEST_MAIN(tst_QTemporaryFile)
 #include "tst_qtemporaryfile.moc"
