@@ -1,5 +1,5 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <jni.h>
 
@@ -40,10 +40,12 @@ private slots:
 
     void ctor();
     void callMethodTest();
+    void callMethodThrowsException();
     void callObjectMethodTest();
     void stringConvertionTest();
     void compareOperatorTests();
     void className();
+    void callStaticMethodThrowsException();
     void callStaticObjectMethodClassName();
     void callStaticObjectMethod();
     void callStaticObjectMethodById();
@@ -77,6 +79,7 @@ private slots:
     void getStaticIntField();
     void getStaticByteFieldClassName();
     void getStaticByteField();
+    void getStaticBooleanField();
     void getStaticLongFieldClassName();
     void getStaticLongField();
     void getStaticDoubleFieldClassName();
@@ -112,9 +115,11 @@ private slots:
     void templateApiCheck();
     void isClassAvailable();
     void fromLocalRef();
+    void largeObjectArray();
 
     void callback_data();
     void callback();
+    void callStaticOverloadResolution();
 
     void cleanupTestCase();
 };
@@ -259,6 +264,15 @@ void tst_QJniObject::callMethodTest()
     }
 }
 
+void tst_QJniObject::callMethodThrowsException()
+{
+    QtJniTypes::QtJniObjectTestClass instance;
+    QTest::ignoreMessage(QtWarningMsg, QRegularExpression("java.lang.Exception"));
+    auto res = instance.callMethod<jobject>("callMethodThrowsException");
+    QVERIFY(!res.isValid());
+    QVERIFY(!QJniEnvironment().checkAndClearExceptions());
+}
+
 void tst_QJniObject::callObjectMethodTest()
 {
     const QString qString = QLatin1String("Hello, Java");
@@ -306,7 +320,7 @@ void tst_QJniObject::compareOperatorTests()
     QJniObject stringObject2 = QJniObject::fromString(str);
     QVERIFY(stringObject != stringObject2);
 
-    jstring jstrobj = 0;
+    jstring jstrobj = nullptr;
     QJniObject invalidStringObject;
     QVERIFY(invalidStringObject == jstrobj);
 
@@ -336,6 +350,15 @@ void tst_QJniObject::className()
     }
 }
 
+void tst_QJniObject::callStaticMethodThrowsException()
+{
+    QTest::ignoreMessage(QtWarningMsg, QRegularExpression("java.lang.Exception"));
+    auto res = QtJniTypes::QtJniObjectTestClass::callStaticMethod<jobject>(
+            "callStaticMethodThrowsException");
+    QVERIFY(!res.isValid());
+    QVERIFY(!QJniEnvironment().checkAndClearExceptions());
+}
+
 void tst_QJniObject::callStaticObjectMethodClassName()
 {
     QJniObject formatString = QJniObject::fromString(QLatin1String("test format"));
@@ -936,6 +959,12 @@ void tst_QJniObject::getStaticByteField()
     QCOMPARE(e, Enum::MAX_VALUE);
 }
 
+void tst_QJniObject::getStaticBooleanField()
+{
+    QCOMPARE(TestClass::getStaticField<jboolean>("S_BOOLEAN_VAR"),
+             TestClass::getStaticField<bool>("S_BOOLEAN_VAR"));
+}
+
 void tst_QJniObject::getStaticLongFieldClassName()
 {
     jlong i = QJniObject::getStaticField<jlong>("java/lang/Long", "MAX_VALUE");
@@ -1034,19 +1063,20 @@ void tst_QJniObject::getStaticCharField()
 
 void tst_QJniObject::getBooleanField()
 {
-    QJniObject obj("org/qtproject/qt/android/QtActivityDelegate");
+    QJniObject obj(testClassName);
 
     QVERIFY(obj.isValid());
-    QVERIFY(!obj.getField<jboolean>("m_backKeyPressedSent"));
+    QVERIFY(obj.getField<jboolean>("BOOL_FIELD"));
+    QVERIFY(obj.getField<bool>("BOOL_FIELD"));
 }
 
 void tst_QJniObject::getIntField()
 {
-    QJniObject obj("org/qtproject/qt/android/QtActivityDelegate");
+    QJniObject obj(testClassName);
 
     QVERIFY(obj.isValid());
-    jint res = obj.getField<jint>("m_currentRotation");
-    QCOMPARE(res, -1);
+    jint res = obj.getField<jint>("INT_FIELD");
+    QCOMPARE(res, 123);
 }
 
 template <typename T>
@@ -1109,6 +1139,7 @@ void tst_QJniObject::setCharField()
 void tst_QJniObject::setBooleanField()
 {
     setField("BOOLEAN_VAR", jboolean(true));
+    setField("BOOLEAN_VAR", true);
 }
 
 void tst_QJniObject::setObjectField()
@@ -1191,6 +1222,7 @@ void tst_QJniObject::setStaticCharField()
 void tst_QJniObject::setStaticBooleanField()
 {
     setStaticField("S_BOOLEAN_VAR", jboolean(true));
+    setStaticField("S_BOOLEAN_VAR", true);
 }
 
 void tst_QJniObject::setStaticObjectField()
@@ -1495,6 +1527,18 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = TestClass::callStaticMethod<jobject[]>("staticObjectArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+
+        QJniArray<jobject> newArray(QList<QJniObject>{QJniObject::fromString(u"one"_s),
+                                                      QJniObject::fromString(u"two"_s),
+                                                      QJniObject::fromString(u"three"_s)});
+        QVERIFY(newArray.isValid());
+        const auto reverse = TestClass::callStaticMethod<jobject[]>("staticReverseObjectArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.size(), 3);
+        QCOMPARE(QJniObject(reverse.at(0)).toString(), u"three"_s);
+        QCOMPARE(QJniObject(reverse.at(1)).toString(), u"two"_s);
+        QCOMPARE(QJniObject(reverse.at(2)).toString(), u"one"_s);
     }
 
     {
@@ -1503,6 +1547,19 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = testClass.callMethod<jobject[]>("objectArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+
+        QJniArray<jobject> newArray(QList<QJniObject>{QJniObject::fromString(u"one"_s),
+                                                      QJniObject::fromString(u"two"_s),
+                                                      QJniObject::fromString(u"three"_s)});
+        QVERIFY(newArray.isValid());
+        const auto reverse = testClass.callMethod<jobject[]>("reverseObjectArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.size(), 3);
+        // QJniArray::at returns a jobject that's a local reference; make sure we don't free it twice
+        QCOMPARE(QJniObject::fromLocalRef(reverse.at(0)).toString(), u"three"_s);
+        QCOMPARE(QJniObject::fromLocalRef(reverse.at(1)).toString(), u"two"_s);
+        QCOMPARE(QJniObject::fromLocalRef(reverse.at(2)).toString(), u"one"_s);
     }
 
     // jbooleanArray ------------------------------------------------------------------------------
@@ -1513,6 +1570,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = TestClass::callStaticMethod<jboolean[]>("staticBooleanArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jboolean>{true, true, true}));
+
+        QJniArray<jboolean> newArray(QList<jboolean>{true, false, false});
+        QVERIFY(newArray.isValid());
+        const auto reverse = TestClass::callStaticMethod<jboolean[]>("staticReverseBooleanArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jboolean>{false, false, true}));
     }
 
     {
@@ -1521,6 +1586,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = testClass.callMethod<jboolean[]>("booleanArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jboolean>{true, true, true}));
+
+        QJniArray<jboolean> newArray(QList<jboolean>{true, false, false});
+        QVERIFY(newArray.isValid());
+        const auto reverse = testClass.callMethod<jboolean[]>("reverseBooleanArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jboolean>{false, false, true}));
     }
 
     // jbyteArray ---------------------------------------------------------------------------------
@@ -1531,6 +1604,18 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = TestClass::callStaticMethod<jbyte[]>("staticByteArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), "abc");
+
+        QJniArray<jbyte> newArray(QByteArray{"cba"});
+        QVERIFY(newArray.isValid());
+        const auto reverse = TestClass::callStaticMethod<jbyte[]>("staticReverseByteArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), "abc");
+
+        const QByteArray reverse2 = TestClass::callStaticMethod<QByteArray>("staticReverseByteArray",
+                                                                            QByteArray("abc"));
+        QCOMPARE(reverse2, "cba");
     }
 
     {
@@ -1539,6 +1624,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = testClass.callMethod<jbyte[]>("byteArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), "abc");
+
+        QJniArray<jbyte> newArray = QJniArrayBase::fromContainer(QByteArray{"cba"});
+        QVERIFY(newArray.isValid());
+        const auto reverse = testClass.callMethod<jbyte[]>("reverseByteArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), "abc");
     }
 
     // jcharArray ---------------------------------------------------------------------------------
@@ -1549,6 +1642,18 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = TestClass::callStaticMethod<jchar[]>("staticCharArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jchar>{u'a', u'b', u'c'}));
+
+        QJniArray<jchar> newArray = {u'c', u'b', u'a'};
+        QVERIFY(newArray.isValid());
+        const auto reverse = TestClass::callStaticMethod<jchar[]>("staticReverseCharArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jchar>{u'a', u'b', u'c'}));
+
+        const QList<jchar> reverse2 = TestClass::callStaticMethod<QList<jchar>>("staticReverseCharArray",
+                                                                        (QList<jchar>{u'c', u'b', u'a'}));
+        QCOMPARE(reverse2, (QList<jchar>{u'a', u'b', u'c'}));
     }
 
     {
@@ -1557,6 +1662,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = testClass.callMethod<jchar[]>("charArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jchar>{u'a', u'b', u'c'}));
+
+        QJniArray<jchar> newArray = {u'c', u'b', u'a'};
+        QVERIFY(newArray.isValid());
+        const auto reverse = testClass.callMethod<jchar[]>("reverseCharArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jchar>{u'a', u'b', u'c'}));
     }
 
     // jshortArray --------------------------------------------------------------------------------
@@ -1567,6 +1680,18 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = TestClass::callStaticMethod<jshort[]>("staticShortArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jshort>{3, 2, 1}));
+
+        QJniArray<jshort> newArray = {3, 2, 1};
+        QVERIFY(newArray.isValid());
+        const auto reverse = TestClass::callStaticMethod<jshort[]>("staticReverseShortArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jshort>{1, 2, 3}));
+
+        const QList<jshort> reverse2 = TestClass::callStaticMethod<QList<jshort>>("staticReverseShortArray",
+                                                                                  (QList<jshort>{1, 2, 3}));
+        QCOMPARE(reverse2, (QList<jshort>{3, 2, 1}));
     }
 
     {
@@ -1575,6 +1700,15 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = testClass.callMethod<jshort[]>("shortArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jshort>{3, 2, 1}));
+
+        QJniArray<jshort> newArray = {3, 2, 1};
+        static_assert(std::is_same_v<decltype(newArray)::Type, jshort>);
+        QVERIFY(newArray.isValid());
+        const auto reverse = testClass.callMethod<jshort[]>("reverseShortArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jshort>{1, 2, 3}));
     }
 
     // jintArray ----------------------------------------------------------------------------------
@@ -1585,6 +1719,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = TestClass::callStaticMethod<jint[]>("staticIntArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jint>{3, 2, 1}));
+
+        QJniArray<jint> newArray = {3, 2, 1};
+        QVERIFY(newArray.isValid());
+        const auto reverse = TestClass::callStaticMethod<jint[]>("staticReverseIntArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jint>{1, 2, 3}));
     }
 
     {
@@ -1593,6 +1735,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = testClass.callMethod<jint[]>("intArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jint>{3, 2, 1}));
+
+        QJniArray<jint> newArray = {3, 2, 1};
+        QVERIFY(newArray.isValid());
+        const auto reverse = testClass.callMethod<jint[]>("reverseIntArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jint>{1, 2, 3}));
     }
 
     // jlongArray ---------------------------------------------------------------------------------
@@ -1603,6 +1753,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = TestClass::callStaticMethod<jlong[]>("staticLongArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jlong>{3, 2, 1}));
+
+        QJniArray<jlong> newArray = {3, 2, 1};
+        QVERIFY(newArray.isValid());
+        const auto reverse = TestClass::callStaticMethod<jlong[]>("staticReverseLongArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jlong>{1, 2, 3}));
     }
 
     {
@@ -1611,6 +1769,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = testClass.callMethod<jlong[]>("longArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jlong>{3, 2, 1}));
+
+        QJniArray<jlong> newArray = {3, 2, 1};
+        QVERIFY(newArray.isValid());
+        const auto reverse = testClass.callMethod<jlong[]>("reverseLongArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jlong>{1, 2, 3}));
     }
 
     // jfloatArray --------------------------------------------------------------------------------
@@ -1621,6 +1787,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = TestClass::callStaticMethod<jfloat[]>("staticFloatArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jfloat>{1.0f, 2.0f, 3.0f}));
+
+        QJniArray<jfloat> newArray = {3.0f, 2.0f, 1.0f};
+        QVERIFY(newArray.isValid());
+        const auto reverse = TestClass::callStaticMethod<jfloat[]>("staticReverseFloatArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jfloat>{1.0f, 2.0f, 3.0f}));
     }
 
     {
@@ -1629,6 +1803,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = testClass.callMethod<jfloat[]>("floatArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jfloat>{1.0f, 2.0f, 3.0f}));
+
+        QJniArray<jfloat> newArray = {3.0f, 2.0f, 1.0f};
+        QVERIFY(newArray.isValid());
+        const auto reverse = testClass.callMethod<jfloat[]>("reverseFloatArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jfloat>{1.0f, 2.0f, 3.0f}));
     }
 
     // jdoubleArray -------------------------------------------------------------------------------
@@ -1639,6 +1821,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = TestClass::callStaticMethod<jdouble[]>("staticDoubleArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jdouble>{3.0, 2.0, 1.0}));
+
+        QJniArray<jdouble> newArray = {3.0, 2.0, 1.0};
+        QVERIFY(newArray.isValid());
+        const auto reverse = TestClass::callStaticMethod<jdouble[]>("staticReverseDoubleArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jdouble>{1.0, 2.0, 3.0}));
     }
 
     {
@@ -1647,6 +1837,14 @@ void tst_QJniObject::templateApiCheck()
 
         const auto array = testClass.callMethod<jdouble[]>("doubleArrayMethod");
         QVERIFY(array.isValid());
+        QCOMPARE(array.size(), 3);
+        QCOMPARE(array.toContainer(), (QList<jdouble>{3.0, 2.0, 1.0}));
+
+        QJniArray<jdouble> newArray = {3.0, 2.0, 1.0};
+        QVERIFY(newArray.isValid());
+        const auto reverse = testClass.callMethod<jdouble[]>("reverseDoubleArray", newArray);
+        QVERIFY(reverse.isValid());
+        QCOMPARE(reverse.toContainer(), (QList<jdouble>{1.0, 2.0, 3.0}));
     }
 
 }
@@ -1667,20 +1865,66 @@ void tst_QJniObject::fromLocalRef()
         QJniObject o = QJniObject::fromLocalRef(env->FindClass("java/lang/String"));
 }
 
+void tst_QJniObject::largeObjectArray()
+{
+    QJniArray<jobject> newArray(QList<QJniObject>{QJniObject::fromString(u"one"_s),
+                                                    QJniObject::fromString(u"two"_s),
+                                                    QJniObject::fromString(u"three"_s)});
+    QVERIFY(newArray.isValid());
+    const QJniArray<QJniObject> reverse = TestClass::callStaticMethod<jobject[]>(
+                                                            "staticReverseObjectArray", newArray);
+    QVERIFY(reverse.isValid());
+    QCOMPARE(reverse.size(), 3);
+
+    // make sure we don't leak local references
+    for (int i = 0; i < 10000; ++i) {
+        QVERIFY(reverse.at(0).isValid());
+        QVERIFY(reverse.at(1).isValid());
+        QVERIFY(reverse.at(2).isValid());
+    }
+}
+
+enum class CallbackParameterType
+{
+    Object,
+    ObjectRef,
+    String,
+    Byte,
+    Boolean,
+    Int,
+    Double,
+    JniArray,
+    QList,
+    QStringList,
+};
 
 static std::optional<TestClass> calledWithObject;
 static int callbackWithObject(JNIEnv *, jobject, TestClass that)
 {
     calledWithObject.emplace(that);
-    return 42;
+    return int(CallbackParameterType::Object);
 }
 Q_DECLARE_JNI_NATIVE_METHOD(callbackWithObject)
+static int callbackWithObjectRef(JNIEnv *, jobject, const TestClass &that)
+{
+    calledWithObject.emplace(that);
+    return int(CallbackParameterType::ObjectRef);
+}
+Q_DECLARE_JNI_NATIVE_METHOD(callbackWithObjectRef)
+
+static std::optional<QString> calledWithString;
+static int callbackWithString(JNIEnv *, jobject, const QString &string)
+{
+    calledWithString.emplace(string);
+    return int(CallbackParameterType::String);
+}
+Q_DECLARE_JNI_NATIVE_METHOD(callbackWithString)
 
 static std::optional<jbyte> calledWithByte;
 static int callbackWithByte(JNIEnv *, jobject, jbyte value)
 {
     calledWithByte.emplace(value);
-    return 43;
+    return int(CallbackParameterType::Byte);
 }
 Q_DECLARE_JNI_NATIVE_METHOD(callbackWithByte)
 
@@ -1688,7 +1932,7 @@ static std::optional<jbyte> calledWithBoolean;
 static int callbackWithBoolean(JNIEnv *, jobject, bool value)
 {
     calledWithBoolean.emplace(value);
-    return 44;
+    return int(CallbackParameterType::Boolean);
 }
 Q_DECLARE_JNI_NATIVE_METHOD(callbackWithBoolean)
 
@@ -1696,7 +1940,7 @@ static std::optional<int> calledWithInt;
 static int callbackWithInt(JNIEnv *, jobject, int value)
 {
     calledWithInt.emplace(value);
-    return 45;
+    return int(CallbackParameterType::Int);
 }
 Q_DECLARE_JNI_NATIVE_METHOD(callbackWithInt)
 
@@ -1704,88 +1948,159 @@ static std::optional<double> calledWithDouble;
 static int callbackWithDouble(JNIEnv *, jobject, double value)
 {
     calledWithDouble.emplace(value);
-    return 46;
+    return int(CallbackParameterType::Double);
 }
 Q_DECLARE_JNI_NATIVE_METHOD(callbackWithDouble)
 
-enum class CallbackParameterType
+static std::optional<QJniArray<jdouble>> calledWithJniArray;
+static int callbackWithJniArray(JNIEnv *, jobject, const QJniArray<jdouble> &value)
 {
-    Object,
-    Byte,
-    Boolean,
-    Int,
-    Double,
-};
+    calledWithJniArray.emplace(value);
+    return int(CallbackParameterType::JniArray);
+}
+Q_DECLARE_JNI_NATIVE_METHOD(callbackWithJniArray)
+
+static std::optional<QList<double>> calledWithQList;
+static int callbackWithQList(JNIEnv *, jobject, const QList<double> &value)
+{
+    calledWithQList.emplace(value);
+    return int(CallbackParameterType::QList);
+}
+Q_DECLARE_JNI_NATIVE_METHOD(callbackWithQList)
+
+static std::optional<QStringList> calledWithStringList;
+static int callbackWithStringList(JNIEnv *, jobject, const QStringList &value)
+{
+    calledWithStringList.emplace(value);
+    return int(CallbackParameterType::QStringList);
+}
+Q_DECLARE_JNI_NATIVE_METHOD(callbackWithStringList)
 
 void tst_QJniObject::callback_data()
 {
     QTest::addColumn<CallbackParameterType>("parameterType");
-    QTest::addColumn<int>("expectedResult");
 
-    QTest::addRow("Object")     << CallbackParameterType::Object
-                                << callbackWithObject(nullptr, nullptr, {});
-    QTest::addRow("Byte")       << CallbackParameterType::Byte
-                                << callbackWithByte(nullptr, nullptr, {});
-    QTest::addRow("Boolean")    << CallbackParameterType::Boolean
-                                << callbackWithBoolean(nullptr, nullptr, {});
-    QTest::addRow("Int")        << CallbackParameterType::Int
-                                << callbackWithInt(nullptr, nullptr, {});
-    QTest::addRow("Double")     << CallbackParameterType::Double
-                                << callbackWithDouble(nullptr, nullptr, {});
+    QTest::addRow("Object")     << CallbackParameterType::Object;
+    QTest::addRow("ObjectRef")  << CallbackParameterType::ObjectRef;
+    QTest::addRow("String")     << CallbackParameterType::String;
+    QTest::addRow("Byte")       << CallbackParameterType::Byte;
+    QTest::addRow("Boolean")    << CallbackParameterType::Boolean;
+    QTest::addRow("Int")        << CallbackParameterType::Int;
+    QTest::addRow("Double")     << CallbackParameterType::Double;
+    QTest::addRow("JniArray")   << CallbackParameterType::JniArray;
+    QTest::addRow("QList")      << CallbackParameterType::QList;
+    QTest::addRow("QStringList") << CallbackParameterType::QStringList;
 }
 
 void tst_QJniObject::callback()
 {
     QFETCH(const CallbackParameterType, parameterType);
-    QFETCH(const int, expectedResult);
 
     TestClass testObject;
-    QJniEnvironment env;
     int result = -1;
 
     switch (parameterType) {
     case CallbackParameterType::Object:
-        QVERIFY(env.registerNativeMethods(testObject.objectClass(), {
+        QVERIFY(TestClass::registerNativeMethods({
             Q_JNI_NATIVE_METHOD(callbackWithObject)
         }));
         result = testObject.callMethod<int>("callMeBackWithObject", testObject);
         QVERIFY(calledWithObject);
         QCOMPARE(calledWithObject.value(), testObject);
         break;
+    case CallbackParameterType::ObjectRef:
+        QVERIFY(TestClass::registerNativeMethods({
+            Q_JNI_NATIVE_METHOD(callbackWithObjectRef)
+        }));
+        result = testObject.callMethod<int>("callMeBackWithObjectRef", testObject);
+        QVERIFY(calledWithObject);
+        QCOMPARE(calledWithObject.value(), testObject);
+        break;
+    case CallbackParameterType::String:
+        QVERIFY(TestClass::registerNativeMethods({
+            Q_JNI_NATIVE_METHOD(callbackWithString)
+        }));
+        result = testObject.callMethod<int>("callMeBackWithString", QString::number(123));
+        QVERIFY(calledWithString);
+        QCOMPARE(calledWithString.value(), "123");
+        break;
     case CallbackParameterType::Byte:
-        QVERIFY(env.registerNativeMethods(testObject.objectClass(), {
+        QVERIFY(TestClass::registerNativeMethods({
             Q_JNI_NATIVE_METHOD(callbackWithByte)
         }));
-        result = testObject.callMethod<int>("callbackWithByte", jbyte(123));
+        result = testObject.callMethod<int>("callMeBackWithByte", jbyte(123));
         QVERIFY(calledWithByte);
         QCOMPARE(calledWithByte.value(), 123);
         break;
     case CallbackParameterType::Boolean:
-        QVERIFY(env.registerNativeMethods(testObject.objectClass(), {
+        QVERIFY(TestClass::registerNativeMethods({
             Q_JNI_NATIVE_METHOD(callbackWithBoolean)
         }));
-        result = testObject.callMethod<int>("callbackWithBoolean", true);
+        result = testObject.callMethod<int>("callMeBackWithBoolean", true);
         QVERIFY(calledWithBoolean);
         QCOMPARE(calledWithBoolean.value(), true);
         break;
     case CallbackParameterType::Int:
-        QVERIFY(env.registerNativeMethods(testObject.objectClass(), {
+        QVERIFY(TestClass::registerNativeMethods({
             Q_JNI_NATIVE_METHOD(callbackWithInt)
         }));
-        result = testObject.callMethod<int>("callbackWithInt", 12345);
+        result = testObject.callMethod<int>("callMeBackWithInt", 12345);
         QVERIFY(calledWithInt);
         QCOMPARE(calledWithInt.value(), 12345);
         break;
     case CallbackParameterType::Double:
-        QVERIFY(env.registerNativeMethods(testObject.objectClass(), {
+        QVERIFY(TestClass::registerNativeMethods({
             Q_JNI_NATIVE_METHOD(callbackWithDouble)
         }));
-        result = testObject.callMethod<int>("callbackWithDouble", 1.2345);
+        result = testObject.callMethod<int>("callMeBackWithDouble", 1.2345);
         QVERIFY(calledWithDouble);
         QCOMPARE(calledWithDouble.value(), 1.2345);
         break;
+    case CallbackParameterType::JniArray: {
+        QVERIFY(TestClass::registerNativeMethods({
+            Q_JNI_NATIVE_METHOD(callbackWithJniArray)
+        }));
+        const QJniArray<double> doubles = { 1.2, 3.4, 5.6 };
+        result = testObject.callMethod<int>("callMeBackWithJniArray", doubles);
+        QVERIFY(calledWithJniArray);
+        QCOMPARE(calledWithJniArray, doubles);
+        break;
     }
-    QCOMPARE(result, expectedResult);
+    case CallbackParameterType::QList: {
+        QVERIFY(TestClass::registerNativeMethods({
+            Q_JNI_NATIVE_METHOD(callbackWithQList)
+        }));
+        const QList<double> doubles = { 1.2, 3.4, 5.6 };
+        result = testObject.callMethod<int>("callMeBackWithQList", doubles);
+        QVERIFY(calledWithQList);
+        QCOMPARE(calledWithQList.value(), doubles);
+        break;
+    }
+    case CallbackParameterType::QStringList: {
+        QVERIFY(TestClass::registerNativeMethods({
+            Q_JNI_NATIVE_METHOD(callbackWithStringList)
+        }));
+        const QStringList strings = { "one", "two" };
+        result = testObject.callMethod<int>("callMeBackWithStringList", strings);
+        QVERIFY(calledWithStringList);
+        QCOMPARE(calledWithStringList.value(), strings);
+        break;
+    }
+    }
+    QCOMPARE(result, int(parameterType));
+}
+
+// Make sure the new callStaticMethod overload taking a class, return type,
+// and argument as template parameters, doesn't break overload resolution
+// and that the class name doesn't get interpreted as the function name.
+void tst_QJniObject::callStaticOverloadResolution()
+{
+    const QString value = u"Hello World"_s;
+    QJniObject str = QJniObject::fromString(value);
+    const auto result = QJniObject::callStaticMethod<jstring, jstring>(
+            QtJniTypes::Traits<TestClass>::className(),
+            "staticEchoMethod", str.object<jstring>()).toString();
+    QCOMPARE(result, value);
 }
 
 QTEST_MAIN(tst_QJniObject)
