@@ -1,6 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // Copyright (C) 2021 Intel Corporation.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QSignalSpy>
@@ -16,6 +16,8 @@
 #  include <QtCore/private/qmachparser_p.h>
 #endif
 
+using namespace Qt::StringLiterals;
+
 // Helper macros to let us know if some suffixes are valid
 #define bundle_VALID    false
 #define dylib_VALID     false
@@ -128,7 +130,7 @@ static std::unique_ptr<QTemporaryFile> patchElf(const QString &source, ElfPatche
         const char *basename = QTest::currentDataTag();
         if (!basename)
             basename = QTest::currentTestFunction();
-        tmplib.reset(new QTemporaryFile(basename + QString(".XXXXXX" SUFFIX)));
+        tmplib.reset(new QTemporaryFile(QDir::currentPath() + u'/' + basename + u".XXXXXX" SUFFIX ""_s));
         QVERIFY2(tmplib->open(), qPrintable(tmplib->errorString()));
 
         // sanity-check
@@ -472,7 +474,7 @@ static void loadCorruptElfCommonRows()
         memcpy(h, &o, sizeof(o));
     });
     newRow("invalid-word-size", "file is for a different word size", [](H h) {
-        h->e_ident[EI_CLASS] = ELFCLASSNONE;;
+        h->e_ident[EI_CLASS] = ELFCLASSNONE;
     });
     newRow("unknown-word-size", "file is for a different word size", [](H h) {
         h->e_ident[EI_CLASS] |= 0x40;
@@ -858,22 +860,19 @@ void tst_QPluginLoader::loadMachO_data()
 
 #  ifdef Q_PROCESSOR_X86_64
     QTest::newRow("machtest/good.x86_64.dylib") << true;
-    QTest::newRow("machtest/good.i386.dylib") << false;
+    QTest::newRow("machtest/good.arm64.dylib") << false;
     QTest::newRow("machtest/good.fat.no-x86_64.dylib") << false;
-    QTest::newRow("machtest/good.fat.no-i386.dylib") << true;
-#  elif defined(Q_PROCESSOR_X86_32)
-    QTest::newRow("machtest/good.i386.dylib") << true;
+    QTest::newRow("machtest/good.fat.no-arm64.dylib") << true;
+#  elif defined(Q_PROCESSOR_ARM)
+    QTest::newRow("machtest/good.arm64.dylib") << true;
     QTest::newRow("machtest/good.x86_64.dylib") << false;
-    QTest::newRow("machtest/good.fat.no-i386.dylib") << false;
+    QTest::newRow("machtest/good.fat.no-arm64.dylib") << false;
     QTest::newRow("machtest/good.fat.no-x86_64.dylib") << true;
 #  endif
-#  ifndef Q_PROCESSOR_POWER_64
-    QTest::newRow("machtest/good.ppc64.dylib") << false;
-#  endif
 
     QTest::newRow("machtest/good.fat.all.dylib") << true;
     QTest::newRow("machtest/good.fat.stub-x86_64.dylib") << false;
-    QTest::newRow("machtest/good.fat.stub-i386.dylib") << false;
+    QTest::newRow("machtest/good.fat.stub-arm64.dylib") << false;
 
     QDir d(QFINDTESTDATA("machtest"));
     const QStringList badlist = d.entryList(QStringList() << "bad*.dylib");
@@ -901,12 +900,7 @@ void tst_QPluginLoader::loadMachO()
     }
 
     QVERIFY(r.pos > 0);
-    QVERIFY(size_t(r.length) >= sizeof(void*));
     QVERIFY(r.pos + r.length < data.size());
-    QCOMPARE(r.pos & (sizeof(void*) - 1), 0UL);
-
-    void *value = *(void**)(data.constData() + r.pos);
-    QCOMPARE(value, sizeof(void*) > 4 ? (void*)(0xc0ffeec0ffeeL) : (void*)0xc0ffee);
 
     // now that we know it's valid, let's try to make it invalid
     ulong offeredlen = r.pos;
