@@ -1,5 +1,5 @@
 // Copyright (C) 2018 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -42,7 +42,12 @@ private slots:
     void imageConversionOverLargerGamut();
     void imageConversionOverLargerGamut2_data();
     void imageConversionOverLargerGamut2();
-
+    void imageConversionOverAnyGamutFP_data();
+    void imageConversionOverAnyGamutFP();
+    void imageConversionOverAnyGamutFP2_data();
+    void imageConversionOverAnyGamutFP2();
+    void imageConversionOverNonThreeComponentMatrix_data();
+    void imageConversionOverNonThreeComponentMatrix();
     void loadImage();
 
     void primaries();
@@ -61,6 +66,11 @@ private slots:
     void transferFunctionTable();
 
     void description();
+    void whitePoint_data();
+    void whitePoint();
+    void setWhitePoint();
+    void grayColorSpace();
+    void grayColorSpaceEffectivelySRgb();
 };
 
 tst_QColorSpace::tst_QColorSpace()
@@ -178,15 +188,41 @@ void tst_QColorSpace::fromIccProfile_data()
     QTest::addColumn<QString>("testProfile");
     QTest::addColumn<QColorSpace::NamedColorSpace>("namedColorSpace");
     QTest::addColumn<QColorSpace::TransferFunction>("transferFunction");
+    QTest::addColumn<QColorSpace::TransformModel>("transformModel");
+    QTest::addColumn<QColorSpace::ColorModel>("colorModel");
     QTest::addColumn<QString>("description");
 
     QString prefix = QFINDTESTDATA("resources/");
     // Read the official sRGB ICCv2 profile:
     QTest::newRow("sRGB2014 (ICCv2)") << prefix + "sRGB2014.icc" << QColorSpace::SRgb
-                                           << QColorSpace::TransferFunction::SRgb << QString("sRGB2014");
+                                      << QColorSpace::TransferFunction::SRgb
+                                      << QColorSpace::TransformModel::ThreeComponentMatrix
+                                      << QColorSpace::ColorModel::Rgb << QString("sRGB2014");
     // My monitor's profile:
     QTest::newRow("HP ZR30w (ICCv4)") << prefix + "HP_ZR30w.icc" << QColorSpace::NamedColorSpace(0)
-                                      << QColorSpace::TransferFunction::Gamma << QString("HP Z30i");
+                                      << QColorSpace::TransferFunction::Gamma
+                                      << QColorSpace::TransformModel::ThreeComponentMatrix
+                                      << QColorSpace::ColorModel::Rgb << QString("HP Z30i");
+    // A profile to HD TV
+    QTest::newRow("VideoHD") << prefix + "VideoHD.icc" << QColorSpace::NamedColorSpace(0)
+                             << QColorSpace::TransferFunction::Custom
+                             << QColorSpace::TransformModel::ElementListProcessing
+                             << QColorSpace::ColorModel::Rgb << QString("HDTV (Rec. 709)");
+    // sRGB on PCSLab format
+    QTest::newRow("sRGB ICCv4 Appearance") << prefix + "sRGB_ICC_v4_Appearance.icc" << QColorSpace::NamedColorSpace(0)
+                                           << QColorSpace::TransferFunction::Custom
+                                           << QColorSpace::TransformModel::ElementListProcessing
+                                           << QColorSpace::ColorModel::Rgb << QString("sRGB_ICC_v4_Appearance.icc");
+    // Grayscale profile
+    QTest::newRow("sGrey-v4") << prefix + "sGrey-v4.icc" << QColorSpace::NamedColorSpace(0)
+                                << QColorSpace::TransferFunction::SRgb
+                                << QColorSpace::TransformModel::ThreeComponentMatrix
+                                << QColorSpace::ColorModel::Gray << QString("sGry");
+    // CMYK profile
+    QTest::newRow("CGATS compat") << prefix + "CGATS001Compat-v2-micro.icc" << QColorSpace::NamedColorSpace(0)
+                                  << QColorSpace::TransferFunction::Custom
+                                  << QColorSpace::TransformModel::ElementListProcessing
+                                  << QColorSpace::ColorModel::Cmyk << QString("uCMY");
 }
 
 void tst_QColorSpace::fromIccProfile()
@@ -194,10 +230,12 @@ void tst_QColorSpace::fromIccProfile()
     QFETCH(QString, testProfile);
     QFETCH(QColorSpace::NamedColorSpace, namedColorSpace);
     QFETCH(QColorSpace::TransferFunction, transferFunction);
+    QFETCH(QColorSpace::TransformModel, transformModel);
+    QFETCH(QColorSpace::ColorModel, colorModel);
     QFETCH(QString, description);
 
     QFile file(testProfile);
-    file.open(QIODevice::ReadOnly);
+    QVERIFY(file.open(QIODevice::ReadOnly));
     QByteArray iccProfile = file.readAll();
     QColorSpace fileColorSpace = QColorSpace::fromIccProfile(iccProfile);
     QVERIFY(fileColorSpace.isValid());
@@ -206,7 +244,14 @@ void tst_QColorSpace::fromIccProfile()
         QCOMPARE(fileColorSpace, namedColorSpace);
 
     QCOMPARE(fileColorSpace.transferFunction(), transferFunction);
+    QCOMPARE(fileColorSpace.transformModel(), transformModel);
+    QCOMPARE(fileColorSpace.colorModel(), colorModel);
     QCOMPARE(fileColorSpace.description(), description);
+
+    QByteArray iccProfile2 = fileColorSpace.iccProfile();
+    QCOMPARE(iccProfile, iccProfile2);
+    QColorSpace fileColorSpace2 = QColorSpace::fromIccProfile(iccProfile2);
+    QCOMPARE(fileColorSpace2, fileColorSpace);
 }
 
 void tst_QColorSpace::imageConversion_data()
@@ -217,9 +262,10 @@ void tst_QColorSpace::imageConversion_data()
 
     QTest::newRow("sRGB -> Display-P3") << QColorSpace::SRgb << QColorSpace::DisplayP3 << 0;
     QTest::newRow("sRGB -> Adobe RGB") << QColorSpace::SRgb << QColorSpace::AdobeRgb << 2;
-    QTest::newRow("Display-P3 -> sRGB") << QColorSpace::DisplayP3 << QColorSpace::SRgb << 0;
     QTest::newRow("Adobe RGB -> sRGB") << QColorSpace::AdobeRgb << QColorSpace::SRgb << 2;
+    QTest::newRow("Adobe RGB -> Display-P3") << QColorSpace::AdobeRgb << QColorSpace::DisplayP3 << 2;
     QTest::newRow("Display-P3 -> Adobe RGB") << QColorSpace::DisplayP3 << QColorSpace::AdobeRgb << 2;
+    QTest::newRow("Display-P3 -> sRGB") << QColorSpace::DisplayP3 << QColorSpace::SRgb << 0;
     QTest::newRow("sRGB -> sRGB Linear") << QColorSpace::SRgb << QColorSpace::SRgbLinear << 0;
     QTest::newRow("sRGB Linear -> sRGB") << QColorSpace::SRgbLinear << QColorSpace::SRgb << 0;
 }
@@ -261,8 +307,9 @@ void tst_QColorSpace::imageConversion()
     QCOMPARE(testImage.colorSpace(), QColorSpace(fromColorSpace));
     for (int i = 0; i < 256; ++i) {
         QRgb p = testImage.pixel(i, 0);
-        QVERIFY(qAbs(qRed(p) - qGreen(p)) <= tolerance);
         QVERIFY(qAbs(qRed(p) - qBlue(p)) <= tolerance);
+        QVERIFY(qAbs(qRed(p) - qGreen(p)) <= tolerance);
+        QVERIFY(qAbs(qGreen(p) - qBlue(p)) <= tolerance);
         QVERIFY((lastRed   - qRed(p))   <= (tolerance / 2));
         QVERIFY((lastGreen - qGreen(p)) <= (tolerance / 2));
         QVERIFY((lastBlue  - qBlue(p))  <= (tolerance / 2));
@@ -439,7 +486,7 @@ void tst_QColorSpace::imageConversionOverLargerGamut()
     testImage.setColorSpace(csfrom);
     for (int y = 0; y < 256; ++y)
         for (int x = 0; x < 256; ++x)
-            testImage.setPixel(x, y, qRgb(x, y, 0));
+            testImage.setPixel(x, y, qRgb(x, y, qAbs(x - y)));
 
     QImage resultImage = testImage.convertedToColorSpace(csto);
     for (int y = 0; y < 256; ++y) {
@@ -517,6 +564,138 @@ void tst_QColorSpace::imageConversionOverLargerGamut2()
     QVERIFY(resultImage.pixelColor(0, 255).greenF() > 1.0f);
 }
 
+void tst_QColorSpace::imageConversionOverAnyGamutFP_data()
+{
+    QTest::addColumn<QColorSpace::NamedColorSpace>("fromColorSpace");
+    QTest::addColumn<QColorSpace::NamedColorSpace>("toColorSpace");
+
+    QTest::newRow("sRGB -> Display-P3") << QColorSpace::SRgb << QColorSpace::DisplayP3;
+    QTest::newRow("sRGB -> Adobe RGB") << QColorSpace::SRgb << QColorSpace::AdobeRgb;
+    QTest::newRow("sRGB -> ProPhoto RGB") << QColorSpace::SRgb << QColorSpace::ProPhotoRgb;
+    QTest::newRow("Adobe RGB -> sRGB") << QColorSpace::AdobeRgb << QColorSpace::SRgb;
+    QTest::newRow("Adobe RGB -> Display-P3") << QColorSpace::AdobeRgb << QColorSpace::DisplayP3;
+    QTest::newRow("Adobe RGB -> ProPhoto RGB") << QColorSpace::AdobeRgb << QColorSpace::ProPhotoRgb;
+    QTest::newRow("Display-P3 -> sRGB") << QColorSpace::DisplayP3 << QColorSpace::SRgb;
+    QTest::newRow("Display-P3 -> Adobe RGB") << QColorSpace::DisplayP3 << QColorSpace::AdobeRgb;
+    QTest::newRow("Display-P3 -> ProPhoto RGB") << QColorSpace::DisplayP3 << QColorSpace::ProPhotoRgb;
+}
+
+void tst_QColorSpace::imageConversionOverAnyGamutFP()
+{
+    QFETCH(QColorSpace::NamedColorSpace, fromColorSpace);
+    QFETCH(QColorSpace::NamedColorSpace, toColorSpace);
+
+    QColorSpace csfrom(fromColorSpace);
+    QColorSpace csto(toColorSpace);
+    csfrom.setTransferFunction(QColorSpace::TransferFunction::Linear);
+    csto.setTransferFunction(QColorSpace::TransferFunction::Linear);
+
+    QImage testImage(256, 256, QImage::Format_RGBX32FPx4);
+    testImage.setColorSpace(csfrom);
+    for (int y = 0; y < 256; ++y)
+        for (int x = 0; x < 256; ++x)
+            testImage.setPixel(x, y, qRgb(x, y, 0));
+
+    QImage resultImage = testImage.convertedToColorSpace(csto);
+    resultImage.convertToColorSpace(csfrom);
+
+    for (int y = 0; y < 256; ++y) {
+        for (int x = 0; x < 256; ++x) {
+            QCOMPARE(resultImage.pixel(x, y), testImage.pixel(x, y));
+        }
+    }
+}
+
+void tst_QColorSpace::imageConversionOverAnyGamutFP2_data()
+{
+    imageConversionOverAnyGamutFP_data();
+}
+
+void tst_QColorSpace::imageConversionOverAnyGamutFP2()
+{
+    QFETCH(QColorSpace::NamedColorSpace, fromColorSpace);
+    QFETCH(QColorSpace::NamedColorSpace, toColorSpace);
+
+    // Same as imageConversionOverAnyGamutFP but using format switching transform
+    QColorSpace csfrom(fromColorSpace);
+    QColorSpace csto(toColorSpace);
+    csfrom.setTransferFunction(QColorSpace::TransferFunction::Linear);
+    csto.setTransferFunction(QColorSpace::TransferFunction::Linear);
+
+    QImage testImage(256, 256, QImage::Format_RGB32);
+    testImage.setColorSpace(csfrom);
+    for (int y = 0; y < 256; ++y)
+        for (int x = 0; x < 256; ++x)
+            testImage.setPixel(x, y, qRgb(x, y, 0));
+
+    QImage resultImage = testImage.convertedToColorSpace(csto, QImage::Format_RGBX32FPx4);
+    resultImage.convertToColorSpace(csfrom, QImage::Format_RGB32);
+
+    for (int y = 0; y < 256; ++y) {
+        for (int x = 0; x < 256; ++x) {
+            QCOMPARE(resultImage.pixel(x, y), testImage.pixel(x, y));
+        }
+    }
+}
+
+void tst_QColorSpace::imageConversionOverNonThreeComponentMatrix_data()
+{
+    QTest::addColumn<QColorSpace>("fromColorSpace");
+    QTest::addColumn<QColorSpace>("toColorSpace");
+
+    QString prefix = QFINDTESTDATA("resources/");
+    QFile file1(prefix + "VideoHD.icc");
+    QFile file2(prefix + "sRGB_ICC_v4_Appearance.icc");
+    QVERIFY(file1.open(QFile::ReadOnly));
+    QVERIFY(file2.open(QFile::ReadOnly));
+    QByteArray iccProfile1 = file1.readAll();
+    QByteArray iccProfile2 = file2.readAll();
+    QColorSpace hdtvColorSpace = QColorSpace::fromIccProfile(iccProfile1);
+    QColorSpace srgbPcsColorSpace = QColorSpace::fromIccProfile(iccProfile2);
+
+    QTest::newRow("sRGB PCSLab -> sRGB") << srgbPcsColorSpace << QColorSpace(QColorSpace::SRgb);
+    QTest::newRow("sRGB -> sRGB PCSLab") << QColorSpace(QColorSpace::SRgb) << srgbPcsColorSpace;
+    QTest::newRow("HDTV -> sRGB") << hdtvColorSpace << QColorSpace(QColorSpace::SRgb);
+    QTest::newRow("sRGB -> HDTV") << QColorSpace(QColorSpace::SRgb) << hdtvColorSpace;
+    QTest::newRow("sRGB PCSLab -> HDTV") << srgbPcsColorSpace << hdtvColorSpace;
+    QTest::newRow("HDTV -> sRGB PCSLab") << hdtvColorSpace << srgbPcsColorSpace;
+}
+
+void tst_QColorSpace::imageConversionOverNonThreeComponentMatrix()
+{
+    QFETCH(QColorSpace, fromColorSpace);
+    QFETCH(QColorSpace, toColorSpace);
+    QVERIFY(fromColorSpace.isValid());
+    QVERIFY(toColorSpace.isValidTarget());
+
+    QVERIFY(!fromColorSpace.transformationToColorSpace(toColorSpace).isIdentity());
+
+    QImage testImage(256, 256, QImage::Format_RGBX64);
+    testImage.setColorSpace(fromColorSpace);
+    for (int y = 0; y < 256; ++y)
+        for (int x = 0; x < 256; ++x)
+            testImage.setPixel(x, y, qRgb(x, y, 0));
+
+    QImage resultImage = testImage.convertedToColorSpace(toColorSpace);
+    QCOMPARE(resultImage.size(), testImage.size());
+    for (int y = 0; y < 256; ++y) {
+        int lastRed = 0;
+        for (int x = 0; x < 256; ++x) {
+            QRgb p = resultImage.pixel(x, y);
+            QVERIFY(qRed(p) >= lastRed);
+            lastRed = qRed(p);
+        }
+    }
+    for (int x = 0; x < 256; ++x) {
+        int lastGreen = 0;
+        for (int y = 0; y < 256; ++y) {
+            QRgb p = resultImage.pixel(x, y);
+            QVERIFY(qGreen(p) >= lastGreen);
+            lastGreen = qGreen(p);
+        }
+    }
+}
+
 void tst_QColorSpace::loadImage()
 {
     QString prefix = QFINDTESTDATA("resources/");
@@ -694,10 +873,28 @@ void tst_QColorSpace::changePrimaries()
     cs.setPrimaries(QColorSpace::Primaries::DciP3D65);
     QVERIFY(cs.isValid());
     QCOMPARE(cs, QColorSpace(QColorSpace::DisplayP3));
+    QCOMPARE(cs.transformModel(), QColorSpace::TransformModel::ThreeComponentMatrix);
     cs.setTransferFunction(QColorSpace::TransferFunction::Linear);
     cs.setPrimaries(QPointF(0.3127, 0.3290), QPointF(0.640, 0.330),
                     QPointF(0.3000, 0.6000), QPointF(0.150, 0.060));
     QCOMPARE(cs, QColorSpace(QColorSpace::SRgbLinear));
+
+
+    QFile iccFile(QFINDTESTDATA("resources/") + "VideoHD.icc");
+    QVERIFY(iccFile.open(QFile::ReadOnly));
+    QByteArray iccData = iccFile.readAll();
+    QColorSpace hdtvColorSpace = QColorSpace::fromIccProfile(iccData);
+    QVERIFY(hdtvColorSpace.isValid());
+    QCOMPARE(hdtvColorSpace.transformModel(), QColorSpace::TransformModel::ElementListProcessing);
+    QCOMPARE(hdtvColorSpace.primaries(), QColorSpace::Primaries::Custom);
+    QCOMPARE(hdtvColorSpace.transferFunction(), QColorSpace::TransferFunction::Custom);
+    // Unsets both primaries and transferfunction because they were inseparable in element list processing
+    hdtvColorSpace.setPrimaries(QColorSpace::Primaries::SRgb);
+    QVERIFY(!hdtvColorSpace.isValid());
+    hdtvColorSpace.setTransferFunction(QColorSpace::TransferFunction::SRgb);
+    QVERIFY(hdtvColorSpace.isValid());
+    QCOMPARE(hdtvColorSpace.transformModel(), QColorSpace::TransformModel::ThreeComponentMatrix);
+    QCOMPARE(hdtvColorSpace, QColorSpace(QColorSpace::SRgb));
 }
 
 void tst_QColorSpace::transferFunctionTable()
@@ -746,5 +943,92 @@ void tst_QColorSpace::description()
     QCOMPARE(srgb.description(), QLatin1String("Linear sRGB")); // Set to empty returns default behavior
 }
 
+void tst_QColorSpace::whitePoint_data()
+{
+    QTest::addColumn<QColorSpace::NamedColorSpace>("namedColorSpace");
+    QTest::addColumn<QPointF>("whitePoint");
+
+    QTest::newRow("sRGB") << QColorSpace::SRgb << QColorVector::D65Chromaticity();
+    QTest::newRow("Adobe RGB") << QColorSpace::AdobeRgb << QColorVector::D65Chromaticity();
+    QTest::newRow("Display-P3") << QColorSpace::DisplayP3 << QColorVector::D65Chromaticity();
+    QTest::newRow("ProPhoto RGB") << QColorSpace::ProPhotoRgb << QColorVector::D50Chromaticity();
+}
+
+void tst_QColorSpace::whitePoint()
+{
+    QFETCH(QColorSpace::NamedColorSpace, namedColorSpace);
+    QFETCH(QPointF, whitePoint);
+
+    QColorSpace colorSpace(namedColorSpace);
+    QPointF wpt = colorSpace.whitePoint();
+    QCOMPARE_LE(qAbs(wpt.x() - whitePoint.x()), 0.0000001);
+    QCOMPARE_LE(qAbs(wpt.y() - whitePoint.y()), 0.0000001);
+}
+
+void tst_QColorSpace::setWhitePoint()
+{
+    QColorSpace colorSpace(QColorSpace::SRgb);
+    colorSpace.setWhitePoint(QPointF(0.33, 0.33));
+    QCOMPARE_NE(colorSpace, QColorSpace(QColorSpace::SRgb));
+    colorSpace.setWhitePoint(QColorVector::D65Chromaticity());
+    // Check our matrix manipulations returned us to where we came from
+    QCOMPARE(colorSpace, QColorSpace(QColorSpace::SRgb));
+}
+
+void tst_QColorSpace::grayColorSpace()
+{
+    QColorSpace spc;
+    QCOMPARE(spc.colorModel(), QColorSpace::ColorModel::Undefined);
+    QVERIFY(!spc.isValid());
+    spc.setWhitePoint(QColorVector::D65Chromaticity());
+    spc.setTransferFunction(QColorSpace::TransferFunction::SRgb);
+    QVERIFY(spc.isValid());
+    QCOMPARE(spc.colorModel(), QColorSpace::ColorModel::Gray);
+
+    QColorSpace spc2(QColorVector::D65Chromaticity(), QColorSpace::TransferFunction::SRgb);
+    QVERIFY(spc2.isValid());
+    QCOMPARE(spc2.colorModel(), QColorSpace::ColorModel::Gray);
+    QCOMPARE(spc, spc2);
+
+    QImage rgbImage(1, 8, QImage::Format_RGB32);
+    QImage grayImage(1, 255, QImage::Format_Grayscale8);
+    // RGB images can not have gray color space
+    rgbImage.setColorSpace(spc2);
+    grayImage.setColorSpace(spc2);
+    QCOMPARE_NE(rgbImage.colorSpace(), spc2);
+    QCOMPARE(grayImage.colorSpace(), spc2);
+    // But gray images can have RGB color space
+    rgbImage.setColorSpace(QColorSpace::SRgb);
+    grayImage.setColorSpace(QColorSpace::SRgb);
+    QCOMPARE(rgbImage.colorSpace(), QColorSpace(QColorSpace::SRgb));
+    QCOMPARE(grayImage.colorSpace(), QColorSpace(QColorSpace::SRgb));
+
+    // While we can not set a grayscale color space on rgb image, we can convert to one
+    QImage grayImage2 = rgbImage.convertedToColorSpace(spc2);
+    QCOMPARE(grayImage2.colorSpace(), spc2);
+    QCOMPARE(grayImage2.format(), QImage::Format_Grayscale8);
+}
+
+void tst_QColorSpace::grayColorSpaceEffectivelySRgb()
+{
+    // Test grayscale colorspace conversion by making a gray color space that should act like sRGB on gray values.
+    QColorSpace sRgb(QColorSpace::SRgb);
+    QColorSpace sRgbGray(QColorVector::D65Chromaticity(), QColorSpace::TransferFunction::SRgb);
+
+    QImage grayImage1(256, 1, QImage::Format_Grayscale8);
+    QImage grayImage2(256, 1, QImage::Format_Grayscale8);
+    for (int i = 0; i < 256; ++i) {
+        grayImage1.bits()[i] = i;
+        grayImage2.bits()[i] = i;
+    }
+    grayImage1.setColorSpace(sRgb);
+    grayImage2.setColorSpace(sRgbGray);
+
+    QImage rgbImage1 = grayImage1.convertedTo(QImage::Format_RGB32);
+    QImage rgbImage2 = grayImage2.convertedToColorSpace(sRgb, QImage::Format_RGB32);
+
+    QCOMPARE(rgbImage1, rgbImage2);
+}
+
 QTEST_MAIN(tst_QColorSpace)
 #include "tst_qcolorspace.moc"
